

import logging

import yaml
from cloud_admin.systemconnection import SystemConnection
from cloud_utils.log_utils.eulogger import Eulogger
from cloud_utils.log_utils import get_traceback
from cloud_utils.system_utils.machine import Machine
from nephoria.usercontext import UserContext
from nephoria import __DEFAULT_API_VERSION__
from boto3 import set_stream_logger

class SystemConnectionFailure(Exception):
    pass


class TestController(object):
    def __init__(self,
                 hostname=None, username='root', password=None, keypath=None, region=None,
                 proxy_hostname=None, proxy_password=None,
                 clouduser_account='nephotest', clouduser_name='sys_admin', clouduser_credpath=None,
                 clouduser_accesskey=None, clouduser_secretkey=None,
                 cloudadmin_credpath=None, cloudadmin_accesskey=None, cloudadmin_secretkey=None,
                 timeout=10, log_level='DEBUG', log_file=None, log_file_level='DEBUG',
                 environment_file=None, https=True,
                 cred_depot_hostname=None, cred_depot_username='root', cred_depot_password=None,
                 api_version=None):

        """

        :param hostname: CLC ssh hostname
        :param username: CLC ssh username
        :param password: CLC ssh password
        :param proxy_hostname: CLC ssh proxy hostname
        :param proxy_password: CLC ssh proxy password
        :param clouduser_account:
        :param clouduser_name:
        :param clouduser_credpath:
        :param clouduser_accesskey:
        :param clouduser_secretkey:
        :param cloudadmin_credpath:
        :param cloudadmin_accesskey:
        :param cloudadmin_secretkey:
        :param timeout:
        """
        if isinstance(log_level, basestring):
            log_level = getattr(logging, log_level.upper(), logging.DEBUG)
        self.log = Eulogger("TESTER:{0}".format(hostname), stdout_level=log_level,
                            logfile=log_file, logfile_level=log_file_level)
        if not hostname and environment_file:
            try:
                component = self.get_component_from_topology(environment_file, 'clc-1')
                hostname = component['clc-1']
            except KeyError:
                component = self.get_component_from_topology(environment_file,
                                                             'clc')
                hostname = component['clc'][0]
        self.log.identifier = "TESTER:{0}".format(hostname)
        self.log = Eulogger("TESTER:{0}".format(hostname), stdout_level=log_level)
        self._region = region
        self._sysadmin = None
        self._cloudadmin = None
        self._test_user = None
        self._cred_depot = None
        self._default_timeout = timeout
        self._https = https
        self._cloud_admin_connection_info = {}
        self._test_user_connection_info = {}
        api_version = api_version or __DEFAULT_API_VERSION__
        self._system_connection_info = {'hostname': hostname,
                                        'username': username,
                                        'password': password,
                                        'keypath': keypath,
                                        'proxy_hostname': proxy_hostname,
                                        'proxy_password': proxy_password,
                                        'proxy_username': None,
                                        'config_qa': None,
                                        'credpath': cloudadmin_credpath,
                                        'aws_access_key': cloudadmin_accesskey,
                                        'aws_secret_key': cloudadmin_secretkey,
                                        'log_level': log_level,
                                        'boto_debug_level': 0,
                                        'euca_user': 'admin',
                                        'euca_account': 'eucalyptus',
                                        'https': https,
                                        'region_domain': region}

        self._cloud_admin_connection_info = {'aws_account_name': 'eucalyptus',
                                             'aws_user_name': 'admin',
                                             'credpath': cloudadmin_credpath,
                                             'region': self.region,
                                             'aws_access_key': cloudadmin_accesskey,
                                             'aws_secret_key': cloudadmin_secretkey,
                                             'service_connection': self,
                                             'log_level': log_level,
                                             'api_version': api_version,
                                             'https': https}

        self._test_user_connection_info = {'aws_account_name': clouduser_account,
                                           'aws_user_name': clouduser_name,
                                           'credpath': clouduser_credpath,
                                           'aws_access_key': clouduser_accesskey,
                                           'aws_secret_key': clouduser_secretkey,
                                           'region': self.region,
                                           'log_level': log_level,
                                           'api_version': api_version,
                                           'https': https}

        self._cred_depot_connection_info = {'hostname': cred_depot_hostname,
                                            'username': cred_depot_username,
                                            'password': cred_depot_password or password,
                                            'log_level': log_level}

        # TODO ??
        self.test_resources = \
            {
                '_instances': [],
                '_volumes': []
            }

    def __repr__(self):
        try:
            myrepr = "{0}:{1}(sysadmin+eucalyptus/admin)".format(
                self.__class__.__name__,
                self._system_connection_info.get('hostname', ""))
            return myrepr
        except Exception as E:
            self.log.debug(E)
            return str(self.__class__.__name__)

    @property
    def region(self):
        if self._region is None and self.sysadmin is not None:
            try:
                regions = self.sysadmin.ec2_connection.get_all_regions()
                if not regions:
                    self.log.warning('No Regions found?')
                else:
                    region = regions[0]
                    name = region.name
                    if name:
                        name = str(name)
                    self.region = name
            except Exception as RE:
                self.log.error('{0}.\nError while fetching region info:{1}'.format(get_traceback(),
                                                                                   RE))
        return self._region

    @region.setter
    def region(self, value):
        if self._region is not None and self._region != value:
            raise ValueError('Can not change region once it has been set')
        self._cloud_admin_connection_info['region'] = value
        self._test_user_connection_info['region'] = value
        self._region = value

    @property
    def cred_depot(self):
        if not self._cred_depot and self._cred_depot_connection_info.get('hostname'):
            self._cred_depot = Machine(**self._cred_depot_connection_info)
        return self._cred_depot

    @property
    def sysadmin(self):
        if not self._sysadmin:
            if not self._system_connection_info.get('hostname', None):
                return None
            try:
                self._sysadmin = SystemConnection(**self._system_connection_info)
            except Exception as TE:
                self.log.error('{0}\nCould not create sysadmin interface, timed out: "{1}"'
                                  .format(get_traceback(), TE))
                raise TE
        return self._sysadmin

    @property
    def admin(self):
        if not self._cloudadmin:
            conn_info = self._cloud_admin_connection_info
            if (conn_info.get('credpath') or
                (conn_info.get('aws_access_key') and conn_info.get('aws_secret_key'))):
                if conn_info.get('credpath'):
                    conn_info['machine'] = self.cred_depot
            else:
                conn_info['eucarc'] = self.sysadmin.creds
            self._cloudadmin = UserContext(**conn_info)
        return self._cloudadmin

    @property
    def user(self):
        if not self._test_user:
            self._test_user = self.create_user_using_cloudadmin(**self._test_user_connection_info)
        return self._test_user

    @user.setter
    def user(self, user):
        if user is None:
            self._test_user = None
        if isinstance(user, UserContext):
            self._test_user = user
            return
        raise ValueError('Unsupported type for test_user:"{0}", "{1}"'.format(user, type(user)))

    def reset_connections(self):
        self._sysadmin = None
        self._cloudadmin = None
        self._test_user = None

    def get_user_by_name(self, aws_account_name, aws_user_name,
                         machine=None, service_connection=None, path='/',
                         https=None, log_level=None, api_version=None):
        """
        Fetch an existing cloud user and convert into a usercontext object.
        For checking basic existence of a cloud user, use the iam interface instead.
        """
        api_version = api_version or self._test_user_connection_info.get('api_version', None)
        try:
            user = self.admin.iam.get_user_info(user_name=aws_user_name,
                                                delegate_account=aws_account_name)
        except Exception:
            self.log.error('Error fetching "account:{0}, user:{1}" has this user been created '
                           'already?'.format(aws_account_name, aws_user_name))
            raise
        if user:
            return self.create_user_using_cloudadmin(aws_account_name=aws_account_name,
                                                     aws_user_name=aws_user_name,
                                                     machine=machine,
                                                     service_connection=service_connection,
                                                     path=path, https=https, log_level=log_level,
                                                     api_version=api_version)
        else:
            raise ValueError('User info not returned for "account:{0}, user:{1}"'
                             .format(aws_account_name, aws_user_name))


    def create_user_using_cloudadmin(self, aws_account_name=None, aws_user_name='admin',
                                     aws_access_key=None, aws_secret_key=None,
                                     credpath=None, eucarc=None,
                                     machine=None, service_connection=None, path='/',
                                     region=None, https=None, api_version=None,
                                     log_level=None):
        if log_level is None:
            log_level = self.log.stdout_level or 'DEBUG'
        if region is None:
            region = self.region
        if https is None:
            https = self._https
        api_version = api_version or self._test_user_connection_info.get('api_version', None)
        self.log.debug('Attempting to create user with params: account:{0}, name:{1}'
                          'access_key:{2}, secret_key:{3}, credpath:{4}, eucarc:{5}'
                          ', machine:{6}, service_connection:{7}, path:{8}, region:{9},'
                          'loglevel:{10}, https:{11}'
                       .format(aws_account_name, aws_user_name, aws_access_key, aws_secret_key,
                               credpath, eucarc, machine, service_connection, path, region,
                               log_level, https))
        service_connection = service_connection or self.sysadmin
        if eucarc:
            if aws_access_key:
                eucarc.access_key = aws_access_key
            if aws_secret_key:
                eucarc.secret_key = aws_secret_key
            if aws_user_name:
                eucarc.user_name = aws_user_name
            if aws_account_name:
                eucarc.account_name = aws_account_name
            return UserContext(eucarc=eucarc,
                               service_connection=service_connection,
                               log_level=log_level,
                               https=https,
                               api_version=api_version)
        if aws_access_key and aws_secret_key:
            return UserContext(aws_access_key=aws_access_key,
                               aws_secret_key=aws_secret_key,
                               aws_account_name=aws_account_name,
                               aws_user_name=aws_user_name,
                               service_connection=service_connection,
                               log_level=log_level,
                               api_version=api_version,
                               https=https)
        if credpath:
            return UserContext(credpath=credpath,
                               machine=machine,
                               log_level=log_level,
                               api_version=api_version)

        info = self.admin.iam.create_account(account_name=aws_account_name,
                                                  ignore_existing=True)
        if info:
            user = self.admin.iam.create_user(user_name=aws_user_name,
                                              delegate_account=info.get('account_name'),
                                              path=path)
            info.update(user)
        else:
            raise RuntimeError('Failed to create and/or fetch Account:"{0}", for User:"{1}"'
                               .format(aws_account_name, aws_user_name))
        ak = self.admin.iam.get_aws_access_key(user_name=info.get('user_name'),
                                               delegate_account=info.get('account_name'))
        if not ak:
            ak = self.admin.iam.create_access_key(user_name=info.get('user_name'),
                                                  delegate_account=info.get('account_name'))
        try:
            info['access_key_id'] = ak['access_key_id']
        except KeyError:
            err_msg = ('Failed to fetch access key for USER:"{0}", ACCOUNT:"{1}"'
                       .format(aws_user_name, aws_account_name))
            self.log.error('{0}\n{1}'.format(get_traceback(), err_msg))
            raise RuntimeError(err_msg)
        if self.admin.iam.get_all_signing_certs(user_name=info.get('user_name'),
                                                delegate_account=info.get('account_name')):
            certs = True
        else:
            certs = False
        user =  UserContext(aws_access_key=info.get('access_key_id'),
                            aws_secret_key=info.get('secret_access_key'),
                            aws_account_name=info.get('account_name'),
                            aws_user_name=info.get('user_name'),
                            existing_certs=certs,
                            machine=self.sysadmin.clc_machine,
                            service_connection=self.sysadmin,
                            log_level=log_level,
                            api_version=api_version,
                            https=https)
        user._user_info = self.admin.iam.get_user_info(user_name=user.user_name,
                                                       delegate_account=user.account_id)
        return user

    def dump_conn_debug(self, info):
        """
        Helper method to format and display the connection info contained in a specific dict.
        Example: self.dump_conn_debug(self._system_connection_info)
        :param info:  connection dict.
        """
        try:
            self.log.debug(
                    'Connection info:\n{0}'
                    .format("\n".join("{0}:{1}".format(x, y) for x,y in info.iteritems())))
        except Exception as doh:
            self.log.error('{0}\nError attempting to dump connection info:{1}'
                           .format(get_traceback(), doh))

    def set_boto_logger_level(self, level='NOTSET', format_string=None):
        """
        Set the global boto loggers levels to 'level'. ie "DEBUG", "INFO", "CRITICAL"
        default is "NOTSET"
        :param level: string matching logging class levels, or integer representing the equiv value
        :param format_string: logging class formatter string
        """
        level = Eulogger.format_log_level(level, 'NOTSET')
        set_stream_logger('boto', level=level, format_string=None)
        set_stream_logger('boto3', level=level, format_string=None)
        set_stream_logger('botocore', level=level, format_string=None)

    def get_component_from_topology(self, environment_file, component_type=None):
        """
        Reads eucalyptus topology from environment file and returns value of expected component.

        Args:
            environment_file - environment file to extract Eucalyptus topology
            component_type - type of the component that needs to be extracted
        Returns:
            a dict with component_type as key and value from the environment file
        """
        try:
            with open(environment_file) as myenv:
                env_dict = yaml.load(myenv)
        except Exception as EE:
            self.log.error('Failed to read env file:"{0}", err:{1}'.format(environment_file, EE))
            raise EE

        result = env_dict['default_attributes']['eucalyptus']['topology'][component_type]

        return {component_type: result}



#!/usr/bin/python

from logging import DEBUG, NOTSET
from boto3 import set_stream_logger
from boto3.session import Session
from nephoria.baseops import BaseOps
from cloud_utils.log_utils.eulogger import Eulogger

class Boto3BaseOps(BaseOps):
    #  The key-name used to find the service url in the runtime-config. ie: EC2_URL, IAM_URL, etc
    EUCARC_URL_NAME = None
    #  Service name/prefix. ie: 'ec2', 's3', 'iam', etc..
    SERVICE_PREFIX = None
    #  Do not set connection class for boto3 ops
    CONNECTION_CLASS = None

    def create_connection_kwargs(self, **kwargs):
        """
        Create connection kwargs for Boto type connections. See Baseops.__init__(**kwargs)
        for kwargs available
        """
        region = kwargs.get('region')
        endpoint_url = self.service_url
        verify = kwargs.get('validate_certs', True)
        api_version = kwargs.get('api_version', "")
        use_ssl = kwargs.get('is_secure', False)
        region = region

        self._connection_kwargs = {'service_name': self.SERVICE_PREFIX,
                                   'aws_access_key_id': self.eucarc.aws_access_key,
                                   'aws_secret_access_key': self.eucarc.aws_secret_key,
                                   'use_ssl': use_ssl,
                                   'region_name': region,
                                   'verify': verify,
                                   'endpoint_url': endpoint_url}
        if api_version is not None:
            self._connection_kwargs['api_version'] = api_version

        return self._connection_kwargs

    def enable_connection_debug(self, level=DEBUG, format_string=None):
        try:
            level = Eulogger.format_log_level(level, 'DEBUG')
            set_stream_logger('botocore', level=level, format_string=None)
        except:
            self.log.error('Could not enable debug for: "{0}"'.format(self))
            raise

    def disable_connection_debug(self, level=NOTSET):
        try:
            self.connection.debug = 0
            level = Eulogger.format_log_level(level, 'NOTSET')
            set_stream_logger('botocore', level=level, format_string=None)
        except:
            self.log.error('Could not disable debug for: "{0}"'.format(self))
            raise

    @property
    def session(self):
        if not self._session:
            if self._user_context:
                self._session = self._user_context.session
            else:
                self._session = Session(aws_access_key_id=self.eucarc.aws_access_key,
                                        aws_secret_access_key=self.eucarc.secret_key,
                                        region_name=self.service_region)
        return self._session

    @session.setter
    def session(self, value):
        if value is None or isinstance(value, Session):
            self._session = value
        else:
            raise TypeError('Expected "{0}" type for session, got: "{1}/{2}"'
                            .format(Session.__class__.__name__, value, type(value)))

    def connect(self, verbose=False,  connection_kwargs=None, connection_method=None):
        """
        Verify the required params have been set, and connect the underlying connection class.

        :param verbose: Dump debug output about the connection
        :param connection_kwargs: options dict containing kwargs used when creating the
                                  underlying connection
        """
        if connection_kwargs:
            self._connection_kwargs = connection_kwargs
        session = None
        if connection_method is None and not self.CONNECTION_CLASS:
            session = self.session
            if not session:
                raise RuntimeError('"{0}". Session was not found/created during connect()'
                                   .format(self))
            connection_method = session.resource
        # Remove and kwargs which are not part of the service connection class creation method
        self._clean_connection_kwargs(connection_kwargs=self._connection_kwargs,
                                      connection_method=connection_method)
        required = []
        for key, value in self._connection_kwargs.iteritems():
            if value is None:
                required.append(key)
        if required:
            self.show_connection_kwargs(connection_kwargs=connection_kwargs)
            raise ValueError('{0}: Required Connection parameters were None: "{1}"'
                             .format(self.__class__.__name__, ", ".join(required)))
        #### Init connection...
        if verbose:
            self.show_connection_kwargs()
        try:
            # Remove any kwargs that are not applicable to this connection class
            # For example 'region' may not be applicable to services such as 'IAM'
            connection_keys = self._connection_kwargs.keys()
            for ckey in connection_keys:
                if ckey not in connection_method.__func__.__code__.co_varnames:
                    self.log.debug('Arg "0" not found in method:"{1}()", removing kwarg '
                                   'connection args.'.format(ckey,
                                                             connection_method.__func__.__name__))
                    self._connection_kwargs.__delitem__(ckey)
            self.connection = connection_method(**self._connection_kwargs)
        except:
            self.show_connection_kwargs()
            raise



def delete_all_stacks(self, timeout=360, poll_sleep=10):
    """
    Deletes all stacks.
    Args:
        timeout: default= 60
        poll_sleep: 10 seconds
    Returns: list of stacks, empty list if succeeded
    """
    stacks = self.describe_stacks()
    poll_count = timeout / poll_sleep
    errors = []
    start = time.time()
    elapsed = 0
    attempts = 0
    if len(stacks) > 0:
        for i in stacks:
            try:
                self.log.debug("Deleting Stack: {0}".format(i))
                self.delete_stack(i)
            except Exception as E:
                errors.append(str(E))
        while elapsed < timeout:

            elapsed = int(time.time() - start)
            attempts += 1
            self.log.debug(sladkfjsadlk)
            try:
                stacks = self.describe_stacks()
                if len(stacks) == 0:
                    break
                for i in stacks:
                    if i in stacks:
                        self.delete_stack(i)
                stacks = self.describe_stacks()
                if stacks:
                    time.sleep(poll_sleep)
            except Clas EE:
                if EE.status == 400:
                    self.log.debug(yay it's deleted')
            except Exception as Doh:
                errors.append(Doh)
    if stacks or errors:
        raise RuntimeError

    return stacks




#!/usr/bin/env python

from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from cloud_utils.log_utils import get_traceback, red, ForegroundColor, BackGroundColor, markup
from cloud_utils.net_utils.remote_commands import RemoteCommands
from cloud_utils.net_utils.sshconnection import SshConnection
from nephoria.testcontroller import TestController
import copy
import time
import os


class SOSReports(CliTestRunner):

    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    _DEFAULT_CLI_ARGS['ticket_number'] = {
        'args': ['--ticket-number'],
        'kwargs': {'dest': 'ticket_number',
                   'help': 'Issue, bug, ticket number or identifier to use (defaults to time)',
                   'default': None}}
    _DEFAULT_CLI_ARGS['timeout'] = {
        'args': ['--timeout'],
        'kwargs': {'dest': 'timeout',
                   'help': 'Timeout for the sos gathering operation',
                   'default': 1200}}

    _DEFAULT_CLI_ARGS['remote_dir'] = {
        'args': ['--remote-dir'],
        'kwargs': {'dest': 'remote_dir',
                   'help': 'Directory on remote host(s)',
                   'default': '/root/'}}

    _DEFAULT_CLI_ARGS['local_dir'] = {
        'args': ['--local-dir'],
        'kwargs': {'dest': 'local_dir',
                   'help': 'Local directory to use for gathering sos reports',
                   'default': ''}}

    _DEFAULT_CLI_ARGS['ip_list'] = {
        'args': ['--ip-list'],
        'kwargs': {'dest': 'ip_list',
                   'help': 'Comma separated list of ips or hostnames to gather sos reports from',
                   'default': None }}

    _DEFAULT_CLI_ARGS['package_url'] = {
        'args': ['--package-url'],
        'kwargs': {'dest': 'package_url',
                   'help': 'Url to use for eucalyptus sos plugin package',
                   'default': "http://downloads.eucalyptus.com/software/tools/centos/6/x86_64/"
                              "eucalyptus-sos-plugins-0.1.5-0.el6.noarch.rpm"}}

    def post_init(self, *args, **kwargs):
        self.start_time = int(time.time())
        self.ticket_number = self.args.ticket_number or self.start_time
        self.remote_dir = os.path.join(self.args.remote_dir,
                                       'euca-sosreport-{0}'.format(self.ticket_number))
        self._ip_list = []


    def _scrub_ip_list(self, value):
        value = value or []
        ip_list = []
        if isinstance(value, basestring):
            value = value.split(',')
        if not isinstance(value, list):
            self.log.error(red('ip_list must be a list of IPs or comma separated string of IPs'))
            raise ValueError('ip_list must be a list of IPs or comma separated string of IPs')
        for ip in value:
            ip_list.append(str(ip).strip())
        return ip_list


    @property
    def ip_list(self):
        if not self._ip_list:
            ip_list = self.args.ip_list or self.tc.sysadmin.eucahosts.keys()
            self._ip_list = self._scrub_ip_list(ip_list)
        return self._ip_list

    @ip_list.setter
    def ip_list(self, value):
        self._ip_list = self._scrub_ip_list(value)

    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            self.log.debug('Attempting to create TestController...')
            tc = TestController(hostname=self.args.clc,
                                environment_file=self.args.environment_file,
                                password=self.args.password,
                                timeout=self.args.timeout,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    @property
    def rc(self):
        rc = getattr(self, '__rc', None)
        if not rc:
            ip_list = self.ip_list
            self.log.debug('Attempting to create remote command driver with ip list: {0}'
                           .format(ip_list))
            rc = RemoteCommands(ips=self.ip_list,
                                username='root',
                                password=self.args.password,
                                timeout=600)
            setattr(self, '__rc', rc)
        return rc

    def clean_method(self):
        pass

    def test1_install_sos_and_plugins(self):
        """
        Attempts to install the SOS and Eucalyptus-sos-plugins on each machine in the cloud.
        """

        rc = self.rc
        rc.results = {}
        self.log.debug('Running install on ips:{0}'.format(rc.ips))
        rc.run_remote_commands(command='yum install sos eucalyptus-sos-plugins -y --nogpg')
        rc.show_results()
        failed = 0
        for host, result in rc.results.iteritems():
            if result.get('status') != 0:
                failed += 1
        if failed:
            raise RuntimeError('{0}/{1} hosts had errors during install sos and plugin packages'
                               .format(failed/len(rc.ips)))


    def test2_run(self):
        """
        Attempts to run SOS on each host in the cloud to create and gather SOS reports.
        """
        command = "mkdir -p " + self.remote_dir
        command += "; sosreport --batch --tmp-dir {0} --ticket-number {1} "\
            .format(self.remote_dir, self.ticket_number)
        rc = self.rc
        rc.results = {}
        rc.run_remote_commands(command=command)
        rc.show_results()
        failed = 0
        for host, result in rc.results.iteritems():
            if result.get('status') != 0:
                failed += 1
        if failed:
            raise RuntimeError('{0}/{1} hosts had errors while attempting to run SOS'
                               .format(failed, len(rc.ips)))


    def test3_download(self):
        """
        Attempts to download the SOS reports from each host in the cloud and store in a local
        directory
        """
        error_msg = ""
        count = 0
        err_count = 0
        host_count = len(self.ip_list)
        for ip in self.ip_list:
            if self.tc:
                if ip in self.tc.sysadmin.eucahosts.keys():
                    host = self.tc.sysadmin.eucahosts.get(ip)
                    ssh = host.ssh
                else:
                    ssh = SshConnection(host=ip, password=self.args.password)
            try:
                remote_tarball_path = ssh.sys("ls -1 {0}/*.xz | grep {1}"
                                               .format(self.remote_dir, self.ticket_number),
                                               code=0)[0]
                tarball_name = os.path.basename(remote_tarball_path)
                local_name = "sosreport-{0}.{1}{2}".format(ip, self.ticket_number,
                                                 tarball_name.split(str(self.ticket_number))[1])
                local_tarball_path = os.path.join(self.args.local_dir, local_name)
                self.log.debug("Downloading file to: " + local_tarball_path)
                ssh.sftp_get(localfilepath=local_tarball_path,
                                  remotefilepath=remote_tarball_path)
            except Exception, e:
                err_count += 1
                msg = '\nError Downloading from: {0}. Error:"{0}"\n'.format(ip, e)
                self.log.error("{0}\n{1}".format(get_traceback(), msg))
                error_msg += msg
            else:
                count += 1
                self.log.info(markup('Downloaded SOS report {0}/{1} to:{2}'
                                     .format(count, host_count, local_tarball_path),
                                     markups=[ForegroundColor.WHITE, BackGroundColor.BG_GREEN]))
        if error_msg:
            self.log.error(red(error_msg))
            raise Exception('Error during download on {0}/{1} hosts'.format(err_count, host_count))


if __name__ == "__main__":
    test =SOSReports()
    result = test.run()
    exit(result)


#!/usr/bin/env python
import re
from boto.exception import S3ResponseError

from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from nephoria.testcontroller import TestController
import copy
import time


class RunInstances(CliTestRunner):

    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            tc = TestController(self.args.clc,
                                password=self.args.password,
                                clouduser_name=self.args.test_user,
                                clouduser_account=self.args.test_account,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    @property
    def user(self):
        user = getattr(self, '__user', None)
        if not user:
            try:
                user = self.tc.get_user_by_name(aws_account_name=self.args.test_account,
                                                aws_user_name=self.args.test_user)
            except:
                user = self.tc.create_user_using_cloudadmin(aws_account_name=self.args.test_account,
                                                            aws_user_name=self.args.test_user)
            setattr(self, '__user', user)
        return user

    @property
    def bucket_prefix(self):
        bucket_prefix = getattr(self, '__bucket_prefix', None)
        if not bucket_prefix:
            bucket_prefix = "nephoria-bucket-test-suite-" + str(int(time.time()))
        return bucket_prefix

    @bucket_prefix.setter
    def bucket_prefix(self, value):
        setattr(self, '__bucket_prefix', value)

    def test_bucket_get_put_delete(self):
        test_bucket = self.bucket_prefix + "-simple-test-bucket"
        bucket = self.tc.user.s3.create_bucket(test_bucket)
        self.tc.user.s3.delete_bucket(test_bucket)

    def test_negative_basic(self):
        """
        Test Coverage:
            - create bucket with empty-string name
            - invalid bucket names
        """
        self.log.debug("Trying to create bucket with empty-string name.")
        try:
            null_bucket_name = ""
            bucket_obj = self.tc.user.s3.create_bucket(null_bucket_name)
            if bucket_obj:
                raise("Should have caught exception for creating bucket with empty-string name.")
        except S3ResponseError as e:
            assert (e.status == 405), 'Expected response status code to be 405, actual status code is ' + str(e.status)
            assert (
            re.search("MethodNotAllowed", e.code)), "Incorrect exception returned when creating bucket with null name."

        self.log.debug("Testing an invalid bucket names, calls should fail.")

        def test_creating_bucket_invalid_names(bad_bucket):
            should_fail = False
            try:
                bucket = self.tc.user.s3.create_bucket(bad_bucket)
                should_fail = True
                try:
                    self.tc.user.s3.delete_bucket(bucket)
                except:
                    self.log.debug("Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail")
            except Exception as e:
                self.log.debug("Correctly caught the exception for bucket name '" + bad_bucket + "' Reason: " + e.reason)
            if should_fail:
                raise("Should have caught exception for bad bucket name: " + bad_bucket)

        # with the EUCA-8864 fix, a new property 'objectstorage.bucket_naming_restrictions'
        # has been introduced, now 'bucket..123', 'bucket.' are actually valid bucket names
        # when using 'extended' naming convention.
        # http://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html
        # when DNS is not being used, for now buckets can be created with bucket
        # names like '/bucket123', 'bucket123/', see EUCA-8863
        # TODO check what bucket naming convention is being used for the test
        for bad_bucket in ["bucket&123", "bucket*123"]:
            test_creating_bucket_invalid_names(self.bucket_prefix + bad_bucket)

    def clean_method(self):
        pass


if __name__ == "__main__":
    test = RunInstances()
    result = test.run()
    exit(result)




from eutester.sshconnection import SshConnection
from eutester.eulogger import Eulogger
import re
import subprocess
keypath = '/root/.ssh/id_rsa.pub'

p = subprocess.Popen('cat {0}'.format(keypath), shell=True,
                     stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
pub_key = p.stdout.read()
result = p.wait()
if result:
    raise RuntimeError('Error reading in key at: "{0}", errcode:{1}'.format(keypath, result))

with open('config_data') as config_data:
    lines = config_data.readlines()

ipset = set([])
ip_regex = re.compile("(^\s*\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})")

for line in lines:
    match = ip_regex.match(line)
    if match:
        ipset.add(match.groups()[0])

logger = Eulogger()

for ip in ipset:
    logger = Eulogger(identifier=ip)
    ssh = SshConnection(host=ip, password='foobar', debugmethod=logger.log.debug)
    ssh.sys("echo " + pub_key + " >> ~/.ssh/authorized_keys", code=0, verbose=True)




#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
#!/usr/bin/env python
from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from nephoria.testcases.euca2ools.euca2ools_image_utils import Euca2oolsImageUtils
from nephoria.usercontext import UserContext
from nephoria.testcontroller import TestController
from cloud_utils.log_utils import get_traceback
from cloud_utils.system_utils import local
from cloud_utils.net_utils.sshconnection import CommandExitCodeException
from boto.s3.bucket import Bucket
from boto.ec2.keypair import KeyPair
from boto.exception import S3ResponseError
import copy
from nephoria.aws.ec2.conversiontask import ConversionTask
from subprocess import CalledProcessError
from urllib2 import Request, urlopen, URLError
from base64 import b64decode
import os
import re
import time
import types


class ImportInstanceTests(CliTestRunner):
    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)
    _DEFAULT_CLI_ARGS['image_url'] = {
        'args': ['--image-url'],
        'kwargs': {'default': None,
                   'help': 'URL containing remote image to create import instance task from'}}

    _DEFAULT_CLI_ARGS['instanceuser'] = {
        'args': ['--instance-user'],
        'kwargs': {'dest': 'instance_user',
                   'default': None,
                   'help': 'Username used for ssh or winrm login ie: Linux:root '
                           'Windows:Administrator'}}

    _DEFAULT_CLI_ARGS['workerip'] = {
        'args': ['--workerip'],
        'kwargs': {'dest': 'worker_machine',
                   'default': None,
                   'help': 'The ip/hostname of the machine that the operation will be performed '
                           'on'}}

    _DEFAULT_CLI_ARGS['worker_username'] = {
        'args': ['--worker-username'],
        'kwargs': {'dest': 'worker_username',
                   'default': 'root',
                   'help': 'The username of the machine that the operation will be performed on'}}

    _DEFAULT_CLI_ARGS['worker_password'] = {
        'args': ['--worker-password'],
        'kwargs': {'dest': 'worker_password',
                   'default': None,
                   'help': 'The password of the machine that the operation will be performed on'}}

    _DEFAULT_CLI_ARGS['worker_keypath'] = {
        'args': ['--worker-keypath'],
        'kwargs': {'dest': 'worker_keypath',
                   'default': None,
                   'help': 'The ssh keypath of the machine that the operation will be performed '
                           'on'}}

    _DEFAULT_CLI_ARGS['destpath'] = {
        'args': ['--destpath'],
        'kwargs': {'default': '/disk1/storage',
                   'help': 'The path on the workip that this operation will be performed on'}}

    _DEFAULT_CLI_ARGS['imagelocation'] = {
        'args': ['--imagelocation'],
        'kwargs': {'default': None,
                   'help': 'The file path on the worker of a pre-existing image to import'}}

    _DEFAULT_CLI_ARGS['urlpass'] = {
        'args': ['--urlpass'],
        'kwargs': {'dest': 'wget_password',
                   'default': None,
                   'help': 'Password needed to retrieve remote url'}}

    _DEFAULT_CLI_ARGS['urluser'] = {
        'args': ['--urluser'],
        'kwargs': {'dest': 'wget_user',
                   'default': None,
                   'help': 'Username needed to retrieve remote url'}}

    _DEFAULT_CLI_ARGS['gigtime'] = {
        'args': ['--gigtime'],
        'kwargs': {'dest': 'time_per_gig',
                   'default': 300,
                   'help': 'Time allowed per gig size of image to be used'}}

    _DEFAULT_CLI_ARGS['virtualization_type'] = {
        'args': ['--virtualization-type'],
        'kwargs': {'default': 'hvm',
                   'help': 'virtualization type hvm or pv'}}

    _DEFAULT_CLI_ARGS['bucket'] = {
        'args': ['--bucket'],
        'kwargs': {'dest': 'bucketname',
                   'default': None,
                   'help': 'bucket name to be used for import task'}}

    _DEFAULT_CLI_ARGS['arch'] = {
        'args': ['--arch'],
        'kwargs': {'dest': 'arch',
                   'default': "x86_64",
                   'help': 'Image architecture ie:x86_64'}}

    _DEFAULT_CLI_ARGS['imageformat'] = {
        'args': ['--imageformat'],
        'kwargs': {'dest': 'imageformat',
                   'default': 'raw',
                   'help': 'image format for import task. ie vmdk raw vhd'}}

    _DEFAULT_CLI_ARGS['platform'] = {
        'args': ['--platform'],
        'kwargs': {'dest': 'platform',
                   'default': "Linux",
                   'help': 'Linux or Windows'}}

    _DEFAULT_CLI_ARGS['uploaded_manifest'] = {
        'args': ['--uploaded-manifest'],
        'kwargs': {'dest': 'upload_manifest',
                   'default': None,
                   'help': 'bucket/prefix location of manifest to register'}}

    _DEFAULT_CLI_ARGS['bundle_manifest'] = {
        'args': ['--bundle-manifest'],
        'kwargs': {'dest': 'bundle_manifest',
                   'default': None,
                   'help': 'file path on worker to bundle manifest to upload'}}

    _DEFAULT_CLI_ARGS['overwrite'] = {
        'args': ['--overwrite'],
        'kwargs': {'action': 'store_true',
                   'default': False,
                   'help': 'Will overwrite files in matching work dir on worker machine if found'}}

    _DEFAULT_CLI_ARGS['time_per_gig'] = {
        'args': ['--time-per-gig'],
        'kwargs': {'default': 100,
                   'help': 'Time allowed (in addition to base timeout) per image size in GB '
                           'before timing out task. Default:100 seconds'}}

    _DEFAULT_CLI_ARGS['base_timeout'] = {
        'args': ['--base-timeout'],
        'kwargs': {'default': 600,
                   'help': 'Base timeout value prior to adding time per gig of image size'}}

    _DEFAULT_CLI_ARGS['task_user_data'] = {
        'args': ['--task-user-data'],
        'kwargs': {'default': '#cloud-config\ndisable_root: false',
                   'help': 'user data to provide to import instance task request'}}

    _DEFAULT_CLI_ARGS['no_clean_on_exit'] = {
        'args': ['--no-clean-on-exit'],
        'kwargs': {'action': 'store_true',
                   'default': False,
                   'help': 'Disable cleanup method upon exit to leave test resources behind'}}

    _DEFAULT_CLI_ARGS['no_https'] = {
        'args': ['--no-https'],
        'kwargs': {'action': 'store_true',
                   'default': False,
                   'help': 'Use http instead of https'}}

    del _DEFAULT_CLI_ARGS['emi']


    def post_init(self):
        """
        cli_test_runner method which runs after __init__()
        """
        self.args.worker_password = self.args.worker_password or self.args.password
        self.args.worker_keypath = self.args.worker_keypath
        # Format platform case sensitive arg.
        if str(self.args.platform).upper().strip() == "WINDOWS":
            self.args.platform = "Windows"
        elif str(self.args.platform).upper().strip() == "LINUX":
            self.args.platform = "Linux"
        if self.args.instance_user is None:
            if self.args.platform == "Windows":
                self.args.instance_user = 'Administrator'
            else:
                self.args.instance_user = 'root'
        self.latest_task_dict = None
        self._user = None
        self._tc = None
        self._image_utils = None
        self._bucket = None
        self._group = None
        self._imagelocation = None
        self._keypair = None
        self._created_keypairs = []
        self._zone = None
        self.args_check = None
        self.current_task = None


    @property
    def imagelocation(self):
        if not self._imagelocation:
            self._imagelocation = self.get_source_volume_image()
        return self._imagelocation

    @property
    def tc(self):
        if not self._tc:
            if not self.args.clc and not self.args.environment_file:
                self.log.error('Must provide --clc or --environment_file arg to run this test')
                raise ValueError('Must provide --clc or --environment_file arg to run this test')
            try:
                self._tc = TestController(hostname=self.args.clc,
                                    environment_file=self.args.environment_file,
                                    password=self.args.password,
                                    clouduser_name=self.args.test_user,
                                    clouduser_account=self.args.test_account,
                                    log_level=self.args.log_level,
                                    https=(not self.args.no_https))
            except Exception as E:
                self.log.error("{0}\nError creating TestController obj:{1}"
                               .format(get_traceback(), E))
                raise E
        return self._tc

    @property
    def created_image(self):
        task = (self.latest_task_dict or {}).get('task', None)
        return getattr(task, 'id', None)

    @property
    def worker_password(self):
        wp = self.args.worker_password or self.args.password
        return wp


    @property
    def worker_keypath(self):
        wk = self.args.worker_keypath or self.args.keypair
        return wk


    @property
    def image_utils(self):
        iu = getattr(self, '_image_utils', None)
        if iu is None:
            # Create an ImageUtils helper from the arguments provided in this testcase...
            setattr(self.args, 'worker_machine', self.tc.sysadmin.clc_machine)
            setattr(self.args, 'user_context', self.user)
            setattr(self.args, 'test_controller', self.tc)
            setattr(self.args, 'user_context', self.user)
            iu = self.do_with_args(Euca2oolsImageUtils)
            setattr(self, '_image_utils', iu)
        return iu


    @property
    def user(self):
        if not self._user:
            if self.args.access_key and self.args.secret_key and self.args.region:
                self._user = UserContext(aws_access_key=self.args.access_key,
                                         aws_secret_key=self.args.secret_key,
                                         region=self.args.region)
            if (self.args.clc or self.args.environment_file) and self.tc:
                self._user = self.tc.user
        return self._user


    def check_url(self, url=None):
        retries = 12
        retry_delay = 10
        req = Request(self.args.image_url)
        url = url or self.args.image_url
        for x in range(retries + 1):
            try:
                response = urlopen(req)
                self.log.debug('URL: "{0}" is valid and reachable!'.format(url))
            except URLError, e:
                if x < retries:
                    if hasattr(e, 'reason'):
                        self.log.debug('Retrying to resolve "{0}", and got: "{1}"'
                                       .format(url, e.reason))
                    elif hasattr(e, 'code'):
                        self.log.debug('Retrying to resolve "{0}", and got: "{1}"'
                                       .format(url, e.code))
                    time.sleep(retry_delay)
                    continue
                else:
                    if hasattr(e, 'reason'):
                        raise AssertionError('INVALID URL: "{0}", "{1}"'
                                             .format(url, e.reason))
                    elif hasattr(e, 'code'):
                        raise AssertionError('INVALID REQUEST: "{0}", "{1}"'
                                             .format(url, e.code))
            break

    @property
    def bucket(self):
        if not self._bucket:
            bucketname = self.args.bucketname
            if not bucketname:
                if self.imagelocation or self.args.image_url:
                    location = self.imagelocation or self.args.image_url
                    image_name = os.path.basename(location)[0:15]
                else:
                    image_name = str(self.args.platform or 'test')
                bucketname = 'eutester_import_' + str(image_name).lower()
            self._bucket = self.user.s3.create_bucket(bucketname)
        return self._bucket

    @bucket.setter
    def bucket(self, value=None):
        if value is None or isinstance(value, Bucket):
            self._bucket = value
        elif isinstance(value, basestring):
            user = self.user
            assert isinstance(user, UserContext)
            try:
                self._bucket = self.user.s3.get_bucket(value)
            except S3ResponseError as SE:
                self.log.error('Error fetching bucket:"{0}", err:"{1}"'.format(value, SE))
                raise SE
        else:
            raise ValueError('Unknown type for bucket: "{0}/{1}"'.format(value, type(value)))


    @classmethod
    def assertEquals(cls, x, y):
        assert x == y, str(x) + ' is not equal to ' + str(y)

    @property
    def keyname(self):
        return self.keypair.name

    @property
    def groupname(self):
        return self.group.name

    @property
    def group(self):
        if not self._group:
            self._group = self._get_security_group()
        return self._group

    @group.setter
    def group(self, group):
        if group is None:
            self._group = group
        else:
            if isinstance(group, basestring):
                group_name = group
            else:
                group_name = group.name
            try:
                self._group = self._get_security_group(group_name=group_name)
            except Exception as E:
                self.log.error("{0}\nError setting up security group, err:{1}"
                               .format(get_traceback(), E))
                raise E

    def _get_security_group(self, group_name=None):
        group_name = group_name or 'import_instance_test_group'
        user = self.user
        group = user.ec2.get_security_group(name=group_name)
        if not group:
            setattr(self, '_created_group', True)
            group = user.ec2.add_group(group_name=group_name)
        #authorize group for ssh and icmp
        user.ec2.authorize_group(group, protocol='tcp', port=22)
        user.ec2.authorize_group(group, protocol='icmp', port='-1')
        if self.args.platform == 'Windows':
            user.ec2.authorize_group(group, protocol='tcp', port='3389')
            user.ec2.authorize_group(group, protocol='tcp', port='80')
            user.ec2.authorize_group(group, protocol='tcp', port='443')
            user.ec2.authorize_group(group, protocol='tcp', port='5985')
            user.ec2.authorize_group(group, protocol='tcp', port='5986')
        return group

    @property
    def keypair(self):
        if not self._keypair:
            try:
                self._keypair = self.user.ec2.create_keypair_and_localcert(
                    "{0}_key_{1}".format(self.name, time.time()))
                self._created_keypairs.append(self._keypair)
            except Exception, ke:
                self.log.error("Failed to find/create a keypair, error:" + str(ke))
                raise ke
        return self._keypair

    @keypair.setter
    def keypair(self, keypair):
        if keypair is None or isinstance(keypair, KeyPair):
            self._keypair = keypair
        else:
            raise ValueError('keypair must be of type None or boto Keypair, got:"{0}/{1}"'
                             .format(keypair, type(keypair)))
    @property
    def zone(self):
        if not self._zone:
            if self.args.zone:
                self._zone = self.args.zone
            else:
                zones = self.tc.admin.ec2.get_zone_names()
                if not zones:
                    raise RuntimeError('Could not find any zones to use in this test')
                self._zone = zones[0]
        return self._zone

    @zone.setter
    def zone(self, zone):
        if zone is None or isinstance(zone, basestring):
            self._zone = zone
        else:
            raise ValueError('Zone must be of type None or String, got: "{0}/{1}"'
                             .format(zone, type(zone)))

    def get_source_volume_image(self, url=None, img_utils=None):
        url = url or self.args.image_url
        img_utils = img_utils or self.image_utils
        if self.args.imagelocation:
            imagelocation = self.args.imagelocation
        else:
            assert isinstance(img_utils, Euca2oolsImageUtils)
            worker = img_utils.worker_machine
            src_img = os.path.basename(url)
            src_img = os.path.join(self.args.destpath, src_img)
            try:
                #Looking for existing file at destpath
                worker.sys('ls ' + src_img, code=0)
            except CommandExitCodeException:
                #File not found at destpath download it...
                worker.wget_remote_image(url=url, dest_file_name=src_img)
            if re.search('\.tar|\.gz|\.xz', src_img):
                src_img = self.unzip_image(image_path=src_img, img_utils=img_utils)
                self.log.debug('Got src image after unzip operation:"{0}"'.format(src_img   ))
            imagelocation = src_img
        return imagelocation

    def unzip_image(self, image_path, img_utils=None, timeout=300):
        img_utils = img_utils or self.image_utils
        worker = img_utils.worker_machine
        filename = os.path.basename(image_path).strip()
        dirname = os.path.dirname(image_path).strip()
        if not dirname:
            dirname = worker.sys('pwd')
            if dirname:
                dirname = dirname[0]
        if re.search('.gz$', filename):
            try:
                worker.sys('which gzip', code=0)
            except CommandExitCodeException:
                self.log.info('"gzip" not found on worker:{0}, installing now...'.format(worker))
                worker.package_manager.install('gzip')
        if re.search('.xz$', filename):
            try:
                worker.sys('which xz', code=0)
            except CommandExitCodeException:
                self.log.info('"xz" not found on worker:{0}, installing now...'.format(worker))
                if re.search('centos|rhel|redhat', worker.distro):
                    worker.package_manager.install('xz')
                else:
                    worker.package_manager.install('xz-utils')
        if re.search('.tar', filename):
            try:
                worker.sys('which tar', code=0)
            except:
                self.log.info('"tar" not found on worker:{0}, installing now...'.format(worker))
                worker.package_manager.install('tar')
            if str(filename).endswith('tar.gz') or str(filename).endswith('tar.xz'):
                tar_flags = '-xvzf'
            elif str(filename).endswith('tar.gz') or str(filename).endswith('tar.xz'):
                tar_flags = '-xvJf'
            else:
                tar_flags = '-xvf'
            dest_name = worker.sys('tar {0} {1} -C {2}'.format(tar_flags, image_path, dirname),
                                  code=0, timeout=timeout)
            if dest_name:
                dest_name = str(dest_name[0]).strip()
            else:
                raise ValueError('File resulting from tar operation not found?')
            dest_path = os.path.join(dirname, dest_name)
            return dest_path
        elif str(filename).endswith('.gz'):
            dest_name = image_path.strip('.gz')
            dest_path = os.path.join(dirname, dest_name)
            worker.sys('gzip -dc < {0} > {1}'.format(image_path, dest_name))
            return dest_path
        elif str(filename).endswith('.xz'):
            dest_name = image_path.strip('.xz')
            dest_path = os.path.join(dirname, dest_name)
            worker.sys('xz {0} -c > {1}'.format(image_path, dirname), code=0,
                       timeout=timeout)
            return dest_path
        else:
            raise ValueError('Unknown file type to unzip:"{0}"'.format(image_path))




    

    def test1_basic_create_import_instance(self,
                                          base_timout=None,
                                          time_per_gig=None):
        '''
        Definition: Attempts to run, monitor and validate the outcome of a
        basic import instance task.
        Will test the following:

        ## TASK CHECKS:
        -Euca2ools import task request
        -Will monitor task and describe task responses until complete, or a
         given timeout is reached.
        -Upon completion will validate task status

        ## TASK INSTANCE CHECKS:
        -Instance status
        -Will Monitor instance to running, then attempt to ping and ssh.
        -Instance params (security group, key, zone, etc)
        -Will request the instance from the system to confirm it is visible
        to this account, etc..

        ## TASK VOLUME CHECKS:
        -Volume status as created and available post task
        -Request the volume from the system to confirm it is visible to this
        account, etc..
        -Volume Params are correct, size, zone, etc per task request

        ## TASK SNAPSHOT CHECKS:
        -Snapshot status completed
        -Verify the owner id is the same as the account id that made the task
        request

        ## TASK IMAGE CHECKS:
        -Verify the image is not public
        -Verify the image state is 'available'
        -Verify the image owner id is the same as the account id that made the
        task request.
        '''
        self.log.info('Attempting euca-import-instance...')
        try:
            self.user.iam.show_user_summary()
            self.user.show()
        except:
            pass
        base_timout = base_timout or self.args.base_timeout
        time_per_gig = time_per_gig or self.args.time_per_gig
        img_utils = self.image_utils
        user = self.user
        assert isinstance(img_utils, Euca2oolsImageUtils)
        params = {'import_file':self.imagelocation,
                  'bucket':self.bucket.name,
                  'zone':self.zone,
                  'format':self.args.imageformat,
                  'instance_type':self.args.vmtype,
                  'arch':self.args.arch,
                  'keypair':self.keyname,
                  'group':self.groupname,
                  'platform':self.args.platform,
                  'user_context': self.user,
                  'user_data':self.args.task_user_data}
        taskid = img_utils.euca2ools_import_instance(**params)
        task = user.ec2.get_conversion_task(taskid=taskid)
        assert isinstance(task,ConversionTask)
        self.current_task = task
        user.ec2.monitor_conversion_tasks(task,
                                        base_timeout=base_timout,
                                        time_per_gig=time_per_gig)

        # Make sure the task returns an instance, check that the instance for
        # proper run state and param use.
        if not task.instanceid:
            raise RuntimeError('Instance ID not found after task completed, '
                               'status msg:' + str(task.statusmessage))
        inst = user.ec2.get_instances(idstring=task.instanceid)
        if inst:
            inst = inst[0]
            username = self.args.instance_user
            euinst = user.ec2.convert_instance_to_euinstance(instance=inst,
                                                             keypair=self.keypair,
                                                             username=username,
                                                             systemconnection=self.tc.sysadmin,
                                                             auto_connect=False)
            user.ec2.monitor_euinstances_to_running(euinst)
            if euinst.platform == 'windows':
                euinst.connect_to_instance(wait_for_boot=180, timeout=300)
            else:
                euinst.connect_to_instance()
        else:
            raise RuntimeError('Instance:"{0}" not found from task:"{1}"'
                            .format(task.instanceid, task.id))
        if not task.image_id:
            raise RuntimeError('Image Id not found after task completed, '
                               'status msg:' + str(task.statusmessage))
        emi = user.ec2.get_emi(emi=task.image_id)
        self.assertEquals(emi.owner_id, user.account_id)
        for snap in task.snapshots:
            self.assertEquals(snap.owner_id, user.account_id)

        self.latest_task_dict = {'params': params,
                                 'task': task,
                                 'instance': euinst}
        return self.latest_task_dict

    def test2_validate_params_against_task(self):
        if not self.latest_task_dict:
            raise RuntimeError('Dict for latest task not found to validate?')
        params = self.latest_task_dict['params']
        task = self.latest_task_dict['task']
        return self.validate_params_against_task(params=params, task=task)

    def test3_make_image_public(self):
        if not self.latest_task_dict:
            raise RuntimeError('Dict for latest task not found to validate?')
        task = self.latest_task_dict['task']
        emi = self.user.ec2.get_emi(emi=task.image_id)
        emi.set_launch_permissions(group_names=['all'])
        self.log.info('\n---------------------------\n'
                      'MADE PUBLIC EMI: {0}'
                      '\n---------------------------'.format(emi.id))

    def test4_tag_image(self):
        if not self.latest_task_dict:
            raise RuntimeError('Dict for latest task not found to validate?')
        task = self.latest_task_dict['task']
        emi = self.user.ec2.get_emi(emi=task.image_id)
        try:
            if self.args.image_url:
                emi.add_tag('source', value=(str(self.args.image_url)))
            emi.add_tag('eutester-created', value="import-instance-test")
        except Exception, te:
            self.log.debug('Could not add tags to image:' + str(emi.id) +
                       ", err:" + str(te))

    def validate_params_against_task(self, params, task):
        assert isinstance(params, types.DictionaryType)
        assert isinstance(task, ConversionTask)
        err_msg = ""
        try:
            if params.has_key('bucket'):
                self.log.debug('Checking task for bucket...')
                #todo put bucket checks here
        except Exception as e:
            self.log.error('Error checking task bucket:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('zone'):
                zone = params['zone']
                self.log.debug('Checking task for zone params:' + str(zone))
                self.assertEquals(zone, task.availabilityzone)
                for volume in task.volumes:
                    self.assertEquals(volume.zone, zone)
                    self.assertEquals(task.instance.placement, zone)
        except Exception as e:
            self.log.error('Error checking task zone:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('size'):
                size = params['size']
                self.log.debug('Checking task for size params:' + str(size))
                self.assertEquals(zone, task.availabilityzone)
                for im_volume in task.importvolumes:
                    self.assertEquals(str(im_volume.volume.size), str(size))
        except Exception as e:
            self.log.error('Error checking task size:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('format'):
                image_format = params['format']
                self.log.debug('Checking task for format:' + str(image_format))
                for volume in task.importvolumes:
                        self.assertEquals(str(volume.image.format).upper(),
                                          str(image_format).upper())
        except Exception as e:
            self.log.error('Error checking task format:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('instance_type'):
                instance_type = params['instance_type']
                self.log.debug('Checking task for instance_type:' + str(instance_type))
                self.assertEquals(task.instance.instance_type, instance_type)
        except Exception as e:
            self.log.error('Error checking task instance-type:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('arch'):
                arch = params['arch']
                self.log.debug('Checking task for arch:' + str(arch))
                emi = self.user.ec2.get_emi(emi=task.image_id)
                self.assertEquals(emi.architecture, arch)
        except Exception as e:
            self.log.error('Error checking task arch:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('keypair'):
                keypair = params['keypair']
                self.log.debug('Checking task for keypair:' + str(keypair))
                self.assertEquals(keypair, task.instance.key_name)
        except Exception as e:
            self.log.error('Error checking task keypair:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('group'):
                group = params['group']
                self.log.debug('Checking task for group:' + str(group))
                ins = self.user.ec2.convert_instance_to_euinstance(
                    task.instance, systemconnection=self.tc.sysadmin, auto_connect=False)
                groups = self.user.ec2.get_instance_security_groups(ins)
                sec_group = groups[0]
                self.assertEquals(sec_group.name, group)
        except Exception as e:
            self.log.error('Error checking task group:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if params.has_key('platform'):
                platform = params['platform']
                self.log.debug('Checking task for platform: ' + str(platform))
                platform = str(platform).lower()
                if platform == 'linux':
                    platform = None
                self.assertEquals(platform, task.instance.platform)
        except Exception as e:
            self.log.error('Error checking task platform:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        try:
            if hasattr(task, 'instanceid') and task.instanceid and \
                    params.has_key('user_data') and params['user_data'] is not None:
                user_data = params['user_data']
                self.log.debug('Checking task for user_data: ' + str(user_data))
                ins_attr = self.user.ec2.connection.get_instance_attribute(
                    task.instanceid, 'userData')
                if 'userData' in ins_attr:
                    if ins_attr['userData'] is None:
                        ins_user_data = ins_attr['userData']
                    else:
                        ins_user_data = b64decode(ins_attr['userData'])
                else:
                    ins_user_data = None
                if not ins_user_data and not user_data:
                    return
                else:
                    self.assertEquals(user_data, ins_user_data)
        except Exception as e:
            self.log.error('Error checking task user-data:\n"{0}'.format(get_traceback()))
            err_msg += str(e) + "\n"

        if err_msg:
            raise Exception("Failures in param validation detected:n\n"
                            + str(err_msg))

    def clean_method(self):
        if self.args.no_clean_on_exit:
            self.log.info('"no_clean_on_exit" set. Skipping Clean method')
            return
        task = self.current_task
        err_buf = ""
        if not task:
            return
        try:
            if task.instance:
                self.user.ec2.terminate_single_instance(task.instance)
        except Exception as E:
            msg = 'Error terminating task instance, err: {0}'.format(E)
            err_buf += msg + "\n"
            self.log.error("{0}\n{1}".format(get_traceback(), msg))
        try:
            if task.volumes:
                self.user.ec2.delete_volumes(task.volumes)
        except Exception as E:
            msg = 'Error deleting task volumes, err: {0}'.format(E)
            err_buf += msg + "\n"
            self.log.error("{0}\n{1}".format(get_traceback(), msg))
        try:
            for keypair in self._created_keypairs:
                self.user.ec2.delete_keypair(keypair)
        except Exception as E:
            msg = 'Error deleting keypair, err: {0}'.format(E)
            err_buf += msg + "\n"
            self.log.error("{0}\n{1}".format(get_traceback(), msg))
        try:
            # Delete the security group only if this test created it
            if self._group and getattr(self, '_created_group', False):
                self.user.ec2.delete_group(self._group)
        except Exception as E:
            msg = 'Error deleting security group, err: {0}'.format(E)
            err_buf += msg + "\n"
            self.log.error("{0}\n{1}".format(get_traceback(), msg))
        if err_buf:
            raise RuntimeError(err_buf)
        else:
            self.log.info('Clean method completed')




if __name__ == "__main__":
    test = ImportInstanceTests()
    result = test.run()
    if test.created_image:
        test.log.info('\n---------------------------\nCreated EMI:{0}\n'
                      '---------------------------'.format(test.created_image))
    exit(result)







# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2016, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import os

from botocore.exceptions import ClientError

from nephoria.baseops.boto3baseops import Boto3BaseOps


class B3_EC2ops(Boto3BaseOps):

    enable_root_user_data = """#cloud-config
disable_root: false"""
    SERVICE_PREFIX = 'ec2'
    EUCARC_URL_NAME = 'ec2_url'
    CONNECTION_CLASS = None

    def setup(self):
        self.key_dir = "./"
        self.local_machine_source_ip = None  # Source ip on local test machine used to reach VMs
        super(B3_EC2ops, self).setup()

    def setup_resource_trackers(self):
        """
        Setup keys in the test_resources hash in order to track artifacts created
        Populate test_resources_clean_methods with resourece type to clean method mappings.
        Note: Some items may have dependencies on other when deleting/removing. Order the list
        in the same order resources should be deleted.
        """
        """
        self.test_resources_clean_methods["instances"] = self.cleanup_test_instances
        self.test_resources_clean_methods["volumes"] = self.clean_up_test_volumes
        self.test_resources_clean_methods["snapshots"] = self.cleanup_test_snapshots
        self.test_resources_clean_methods["keypairs"] = self.delete_ec2_resources
        self.test_resources_clean_methods["security_groups"] = self.delete_ec2_resources
        self.test_resources_clean_methods["images"] = self.delete_ec2_resources
        self.test_resources_clean_methods["addresses"] = self.cleanup_addresses
        self.test_resources_clean_methods["conversion_tasks"] = \
            self.cleanup_conversion_task_resources
        for resource_type in self.test_resources_clean_methods.keys():
            self.test_resources[resource_type] = []
        """

    def create_keypair(self, key_name, key_dir=None, save_to_local=True):
        """
        Creates a key pair and saves the key pair into a local directory.

        Args:
            key_name: name of the key pair
            key_dir: where the key is saved during the test
            save_to_local: if set to True, this method writes the key pair to a file in key_dir

        Returns:
            returns key pair if successfully created
        """
        if not key_dir:
            key_dir = os.getcwd()
        self.log.debug('Creating key {0}'.format(key_name))
        try:
            k = self.connection.create_key_pair(KeyName=key_name)
        except ClientError, e:
            raise Exception(e.message)
        key_file = '{0}/{1}.pem'.format(key_dir, k.key_name)

        if save_to_local:
            with open(key_file, 'w') as key_material:
                key_material.write(k.key_material)
            self.log.debug("Saved key pair at '{0}'".format(key_file))
        try:
            ret_key = self.connection.KeyPair(key_name)
            ret_key.load()
            self.log.debug("Successfully created keypair '{0}'".format(k.key_name, key_file))
            return ret_key
        except ClientError, e:
            raise Exception(e.message)

    def delete_keypair(self, key_name, delete_local=False, key_dir=None):
        """
        Deletes key pair and tests if the key pair's removed.

        Args:
            key_name: name of the key pair to delete
            delete_local: if set to True, if deletes the local file from key_dir
            key_dir: deletes the key pair from key_dir, if delete_local is set to True

        Returns:
            Returns True on success.
        """
        if not key_dir:
            key_dir = os.getcwd()
        k = self.connection.KeyPair(key_name)
        self.log.debug("Deleting key '{0}'".format(key_name))
        try:
            k.load()
            k.delete()
        except ClientError, e:
            raise Exception(e.message)
        try:
            k.reload()
        except:
            return True
                        



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from nephoria.testcases.euca2ools.euca2ools_image_utils import Euca2oolsImageUtils
from nephoria.testcontroller import TestController
from nephoria.usercontext import UserContext
from nephoria.aws.ec2.ec2ops import EC2ResourceNotFoundException
from cloud_utils.log_utils import get_traceback, red
import copy
import os
import time


class Load_Pv_Image(CliTestRunner):

    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    _DEFAULT_CLI_ARGS['test_account'] = {
         'args': ['--test-account'],
         'kwargs': {"help": "Cloud account name to use",
                    "default": "eucalyptus"}}

    _DEFAULT_CLI_ARGS['kernel_image_url'] = {
        'args': ['--kernel-image-url'],
        'kwargs': { 'default': None,
                    'help': 'URL containing the kernel image to be downloaded to the worker '
                            'machine and used in the pv image' }}

    _DEFAULT_CLI_ARGS['kernelfilepath'] = {
        'args': ['--kernelfilepath'],
        'kwargs': { 'default': None,
                    'help': 'An existing file path on the worker machine containing the '
                            'kernel disk image to use' }}

    _DEFAULT_CLI_ARGS['ramdisk_image_url'] = {
        'args': ['--ramdisk-image-url'],
        'kwargs': { 'default': None,
                    'help': 'URL containing the initrd image to be downloaded to the worker '
                            'machine and used in the pv image' }}

    _DEFAULT_CLI_ARGS['ramdiskfilepath'] = {
        'args': ['--ramdiskfilepath'],
        'kwargs': { 'default': None,
                    'help': 'An existing file path on the worker machine containing the '
                            'ramdisk image to use' }}

    _DEFAULT_CLI_ARGS['disk_image_url'] = {
        'args': ['--disk-image-url'],
        'kwargs': { 'default': None,
                    'help': 'URL containing the image to be downloaded to the worker machine '
                            'and used for the pv disk image' }}

    _DEFAULT_CLI_ARGS['diskfilepath'] = {
        'args': ['--diskfilepath'],
        'kwargs': { 'default': None,
                    'help': 'An existing file path on the worker machine containing the '
                            'image to use' }}

    _DEFAULT_CLI_ARGS['workerip'] = {
        'args': ['--workerip'],
        'kwargs': { 'dest': 'worker_machine',
                    'default': None,
                    'help': 'The ip/hostname of the machine that the operation will be'
                            ' performed on' }}

    _DEFAULT_CLI_ARGS['worker_username'] = {
        'args': ['--worker-username'],
        'kwargs': { 'dest': 'worker_username',
                    'default': 'root',
                    'help': 'The username of the machine that the operation will be '
                            'performed on' }}

    _DEFAULT_CLI_ARGS['worker_password'] = {
        'args': ['--worker-password'],
        'kwargs': { 'dest': 'worker_password',
                    'default': None,
                    'help': 'The password of the machine that the operation will be '
                            'performed on' }}

    _DEFAULT_CLI_ARGS['worker_keypath'] = {
        'args': ['--worker-keypath'],
        'kwargs': { 'dest': 'worker_keypath',
                    'default': None,
                    'help': 'The ssh keypath of the machine that the operation will be'
                            ' performed on' }}

    _DEFAULT_CLI_ARGS['destpath'] = {
        'args': ['--destpath'],
        'kwargs': { 'default': '/disk1/storage',
                    'help': 'The path on the workip that this operation will be performed on' }}

    _DEFAULT_CLI_ARGS['urlpass'] = {
        'args': ['--urlpass'],
        'kwargs': { 'dest': 'wget_password',
                    'default': None,
                    'help': 'Password needed to retrieve remote url' }}

    _DEFAULT_CLI_ARGS['urluser'] = {
        'args': ['--urluser'],
        'kwargs': { 'dest': 'wget_user',
                    'default': None,
                    'help': 'Username needed to retrieve remote url' }}

    _DEFAULT_CLI_ARGS['interbundletime'] = {
        'args': ['--interbundletime'],
        'kwargs': { 'dest': 'inter_bundle_timeout',
                    'default': 120,
                    'help': 'Inter-bundle timeout' }}

    _DEFAULT_CLI_ARGS['bucket'] = {
        'args': ['--bucket'],
        'kwargs': { 'dest': 'bucketname',
                    'default': None,
                    'help': 'bucketname' }}

    _DEFAULT_CLI_ARGS['overwrite'] = {
        'args': ['--overwrite'],
        'kwargs': { 'action': 'store_true',
                    'default': False,
                    'help': 'Will overwrite files in matching work dir on worker '
                            'machine if found' }}

    _DEFAULT_CLI_ARGS['no_existing_images'] = {
        'args': ['--no-existing-images'],
        'kwargs': { 'action': 'store_true',
                    'default': False,
                    'help': 'If set this will not use existing images found on the '
                            'system sharing the same image name(s) for kernel(eki) and '
                            'ramdisk(eri) images when building the final image(EMI' }}

    _DEFAULT_CLI_ARGS['time_per_gig'] = {
        'args': ['--time-per-gig'],
        'kwargs': { 'default': 300,
                    'help': 'Time allowed per image size in GB before timing out.' }}

    _DEFAULT_CLI_ARGS['remove_created_images'] = {
        'args': ['--remove-created-images'],
        'kwargs': { 'action': 'store_true',
                    'default': False,
                    'help': 'Flag if set will attempt to deregisteri mages this test created' }}


    def post_init(self, *args, **kwargs):
        self._emi = None
        self.eki = None
        self.eri = None
        self.created_image = None
        self._user = None
        self._tc = None
        self._image_utils = None

        if not self.args.test_list and self.args.emi:
            if (not self.args.disk_image_url and not self.args.diskfilepath) or \
                    (self.args.disk_image_url and self.args.diskfilepath):
                raise ValueError('Must provide "either" a url (image_url) to a disk image or the '
                                 'file path to an existing image on the worker '
                                 'machine (imagefilepath)')
            if (not self.args.kernel_image_url and not self.args.kernelfilepath) or \
                    (self.args.kernel_image_url and self.args.kernelfilepath):
                raise ValueError('Must provide "either" a url (kernel_image_url) to a kernel '
                                 'image or the file path to an existing image on the worker '
                                 'machine (kernelfilepath)')
            if (not self.args.ramdisk_image_url and not self.args.ramdiskfilepath) or \
                    (self.args.ramdisk_image_url and self.args.ramdiskfilepath):
                raise ValueError('Must provide "either" a url (ramdisk_image_url) to a '
                                 'ramdisk image or the file path to an existing image on the worker '
                                 'machine (ramdiskfilepath)')


        self.args.worker_password = self.args.worker_password or self.args.password
        self.args.worker_keypath = self.args.worker_keypath
        self.args.virtualization_type = 'paravirtual'

    @property
    def emi(self):
        if not self._emi:
            if self.args.emi:
                self._emi = self.user.ec2.get_emi(self.args.emi, state=None)
        return self._emi

    @emi.setter
    def emi(self, emi):
        if isinstance(emi, basestring):
            emi = self.user.ec2.get_emi(emi, state=None)
        self._emi = emi


    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            tc = TestController(self.args.clc,
                                environment_file=self.args.environment_file,
                                password=self.args.password,
                                clouduser_name=self.args.test_user,
                                clouduser_account=self.args.test_account,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    @property
    def image_utils(self):
        iu = getattr(self, '_image_utils', None)
        if iu is None:
            if not self.user:
                self.log.warning(red('Cant create image utils w/o user, create user first'))
                return None
            # Create an ImageUtils helper from the arguments provided in this testcase...
            setattr(self.args, 'worker_machine', self.tc.sysadmin.clc_machine)
            setattr(self.args, 'user_context', self.user)
            iu = self.do_with_args(Euca2oolsImageUtils)
            setattr(self, '_image_utils', iu)
        return iu

    @property
    def user(self):
        try:
            if not self._user:
                if self.args.access_key and self.args.secret_key and self.args.region:
                    self._user = UserContext(aws_access_key=self.args.access_key,
                                             aws_secret_key=self.args.secret_key,
                                             region=self.args.region)
                if (self.args.clc or self.args.environment_file) and self.tc:
                    self._user = self.tc.user
        except Exception as UE:
            self.log.error('{0}\nFailed to create user: {1}'.format(get_traceback(), UE))
        return self._user

    def test1_do_kernel_image(self):
        """
        Description:
        Registers a kernel image with the cloud for use in creating an EMI.
        Attempts to either use an existing file path or download from a URL (whichever has been
        provided by the user) to a 'worker machine'. The worker machine will default to the CLC if
        another host is not provided.
        The image is bundled, uploaded and registered using euca2ools on the worker machine.
        """
        size = None
        image_utils = self.image_utils
        kernelfilepath = self.args.kernelfilepath
        kernel_image_url = self.args.kernel_image_url
        filename = os.path.basename(kernelfilepath or kernel_image_url)
        imagename = filename[0:20] + '_by_eutester'
        try:
            image = self.user.ec2.get_emi(emi='ki', filters={'name':imagename})
        except EC2ResourceNotFoundException:
            image = None
        if image:
            if self.args.no_existing_images:
                x = 0
                while True:
                    try:
                         x += 1
                         newname = "{0}_{1}".format(imagename, x)
                         self.user.ec2.get_emi(emi='ki', filters={'name':newname})
                    except EC2ResourceNotFoundException:
                        imagename = newname
                        break
            else:
                self.status('Found existing EKI image:"{0}" with name: "{1}"'
                            .format(image.id, image.name))
                self.eki = image
                self.user.ec2.show_image(self.eki,verbose=True)
                return self.eki
        if not kernelfilepath:
            destpath = self.args.destpath
            size, kernelfilepath = image_utils.wget_image(image_url=kernel_image_url,
                                                          destpath=destpath)
        manifest = image_utils.euca2ools_bundle_image(path=kernelfilepath,
                                                      destination=self.args.destpath)
        upmanifest = image_utils.euca2ools_upload_bundle(manifest=manifest,
                                                         bucketname=imagename + '_eutester_pv')
        eki = image_utils.euca2ools_register(manifest = upmanifest, name= imagename)
        # Make sure this image can be retrieved from the system...
        image = self.user.ec2.get_emi(eki, state=None)
        assert image.id == eki, 'Image retrieved from system did not match the test image id. ' \
                                'Fix the test?'
        # Add some tags to inform the cloud admin/users where this image came from...
        image.add_tag(key='Created by eutester load_pv_image test')
        if size is not None:
            image.add_tag(key='size', value=str(size))
        if kernel_image_url:
            image.add_tag(key='source', value=kernel_image_url)
        image.update()
        self.eki = image
        self.user.ec2.show_image(self.eki, verbose=True)
        return self.eki

    def test2_do_ramdisk_image(self):
        """
        Description:
        Registers a ramdisk image with the cloud for use in creating an EMI.
        Attempts to either use an existing file path or download from a URL (whichever has been
        provided by the user) to a 'worker machine'. The worker machine will default to the CLC if
        another host is not provided.
        The image is bundled, uploaded and registered using euca2ools on the worker machine.
        """
        size = None
        image_utils = self.image_utils
        ramdisk_image_url = self.args.ramdisk_image_url
        ramdiskfilepath = self.args.ramdiskfilepath
        filename =  os.path.basename(ramdiskfilepath or ramdisk_image_url)
        imagename =filename[0:20] + '_by_eutester'
        try:
            image = self.user.ec2.get_emi(emi='ri', filters={'name':imagename})
        except EC2ResourceNotFoundException:
            image = None
        if image:
            if self.args.no_existing_images:
                x = 0
                while True:
                    try:
                         x += 1
                         newname = "{0}_{1}".format(imagename, x)
                         self.user.ec2.get_emi(emi='ri', filters={'name':newname})
                    except EC2ResourceNotFoundException:
                        imagename = newname
                        break
            else:
                self.status('Found existing ERI image:"{0}" with name: "{1}"'
                            .format(image.id, image.name))
                self.eri = image
                self.user.ec2.show_image(self.eri,verbose=True)
                return self.eri
        if not ramdiskfilepath:
            destpath = self.args.destpath
            size, ramdiskfilepath = image_utils.wget_image(image_url=ramdisk_image_url,
                                                           destpath=destpath)
        manifest = image_utils.euca2ools_bundle_image(path=ramdiskfilepath,
                                                      destination=self.args.destpath)
        upmanifest = image_utils.euca2ools_upload_bundle(manifest=manifest,
                                                         bucketname=imagename + '_eutester_pv')
        eri = image_utils.euca2ools_register(manifest = upmanifest, name= imagename)
        # Make sure this image can be retrieved from the system...
        image = self.user.ec2.get_emi(eri, state=None)
        assert image.id == eri, 'Image retrieved from system did not match the test image id. ' \
                                'Fix the test?'
        # Add some tags to inform the cloud admin/users where this image came from...
        image.add_tag(key='Created by eutester load_pv_image test')
        if size is not None:
            image.add_tag(key='size', value=str(size))
        if ramdisk_image_url:
            image.add_tag(key='source', value=ramdisk_image_url)
        image.update()
        self.eri = image
        self.user.ec2.show_image(self.eri, verbose=True)
        return self.eri

    def test3_do_image(self):
        """
        Description:
        Registers an image with the cloud using the ERI, and EKI found or created by this test.
        Attempts to either use an existing file path or download from a URL (whichever has been
        provided by the user) to a 'worker machine'. The worker machine will default to the CLC if
        another host is not provided.
        The image is bundled, uploaded and registered using euca2ools on the worker machine.
        """
        size = None
        image_utils = self.image_utils
        diskfilepath = self.args.diskfilepath
        disk_image_url = self.args.disk_image_url
        filename = os.path.basename(diskfilepath or disk_image_url)
        imagename = filename[0:20] + '_by_eutester'
        if not diskfilepath:
            destpath = self.args.destpath
            size, diskfilepath = image_utils.wget_image(image_url=disk_image_url,
                                                        destpath=destpath)
        try:
            self.user.ec2.get_emi(emi='', filters={'name':imagename}, state=None)
        except EC2ResourceNotFoundException:
            pass
        else:
            # imagename is already taken.
            # Always create a new EMI, so make sure we increment the image name...
            x = 0
            while True:
                try:
                     x += 1
                     newname = "{0}_{1}".format(imagename, x)
                     self.user.ec2.get_emi(emi='', filters={'name':newname})
                     self.log.debug('image name:"{0}" is already in use...'.format(newname))
                except EC2ResourceNotFoundException:
                    imagename = newname
                    self.log.debug('Found an unused image name. Using name:"{0}"'.format(imagename))
                    break
        manifest = image_utils.euca2ools_bundle_image(path=diskfilepath,
                                                      destination=self.args.destpath)
        upmanifest = image_utils.euca2ools_upload_bundle(manifest=manifest,
                                                         bucketname=imagename + '_eutester_pv')
        emi = image_utils.euca2ools_register(manifest = upmanifest,
                                                  name= imagename,
                                                  kernel=self.eki.id,
                                                  ramdisk=self.eri.id,
                                                  description='"created by eutester '
                                                              'load_pv_image test"',
                                                  virtualization_type='paravirtual',
                                                  arch='x86_64'
                                                  )
        # Make sure this image can be retrieved from the system...
        image = self.user.ec2.get_emi(emi, state=None)
        assert image.id == emi, 'Image retrieved from system did not match the test image id. ' \
                                'Fix the test?'
        # Add some tags to inform the cloud admin/users where this image came from...
        image.add_tag(key='eutester-created', value='Created by eutester load_pv_image test')
        if size is not None:
            image.add_tag(key='size', value=str(size))
        if disk_image_url:
            image.add_tag(key='source', value=disk_image_url)
        image.update()
        self.emi = image
        self.user.ec2.show_image(self.emi, verbose=True)
        return self.emi

    def test4_make_image_public(self):
        """
        Description:
        Attempts to set the launch permissions to ALL, making the image public.
        """
        emi_id = self.emi or self.args.emi
        if not emi_id:
            raise SkipTestException('No emi found or provided')
        if not isinstance(emi_id, basestring):
            emi_id = emi_id.id
        emi = self.user.ec2.get_emi(emi_id, state=None)
        emi.set_launch_permissions(group_names=['all'])
        emi.update()
        self.user.ec2.show_image(emi)

    def show_images(self):
        '''
        Attempts to fetch the EMI, EKI, and ERI created by this test and display them in table
        format to the user.
        '''
        self.log.debug('\nCreate the following Image(s)...\n')
        images = []
        if self.emi:
            self.emi.update()
            images.append(self.emi)
        if self.eri:
            self.eri.update()
            images.append(self.eri)
        if self.eki:
            self.eki.update()
            images.append(self.eki)
        if not images:
            self.log.debug('No IMAGES were created?')
        else:
            self.user.ec2.show_images(images=images, verbose=True)
        if not self.emi and self.eri and self.eki:
            self.user.ec2.critical('\nTEST FAILED: Could not find all images (EMI, ERI, EKI)')

    def test5_run_new_pv_image(self):
        """
        Description:
        Attempts to run an instance from the newly created PV image.
        Will attempt to ping/ssh into the instance once running and execute the 'uptime' command.
        """
        emi = self.emi
        if not emi:
            raise SkipTestException('No emi found or provided')
        if isinstance(emi, basestring):
            emi = self.user.ec2.get_emi(emi, state=None)
        self.reservation = None
        ### Add and authorize a group for the instance
        self.group = self.user.ec2.add_group('load_pv_image_test')
        self.user.ec2.authorize_group(self.group, port=22, protocol='tcp')
        self.user.ec2.authorize_group(self.group,  protocol='icmp', port=-1)
        ### Generate a keypair for the instance
        localkeys = self.user.ec2.get_all_current_local_keys()
        if localkeys:
            self.keypair = localkeys[0]
            self.keypair_name = self.keypair.name
        else:
            self.keypair_name = "load_pv_test_keypair" + str(int(time.time()))
            self.keypair = self.user.ec2.get_keypair(key_name=self.keypair_name)
        try:
            size = int(self.emi.tags.get('size', 0)) * int(self.args.time_per_gig)
            timeout = size or 300
            instance = self.user.ec2.run_image(image=emi, keypair=self.keypair,
                                             group=self.group, timeout=timeout)[0]
            instance.sys('uptime', code=0)
            self.status("Run new PV image PASSED")
        finally:
            emi.update()
            self.log.debug('Image states after run attempt:')
            self.show_images()

    def clean_method(self):
        """
        Description:
        Attempts to clean up resources/artifacts created during this test.
        This method will not clean up the images created in this
        test. Will attempt to delete/terminate instances, keypairs, etc..
        """
        pass

if __name__ == "__main__":
    testcase = Load_Pv_Image()
    # Create a single testcase to wrap and run the image creation tasks.
    result = testcase.run()
    if result:
        testcase.log.error('TEST FAILED WITH RESULT:{0}'.format(result))
    else:
        testcase.status('TEST PASSED')
    exit(result)






from prettytable import PrettyTable
from nephoria.testcontroller import TestController
from cloud_admin.backends.network.midget import Midget
from cloud_utils.log_utils import get_traceback, markup
from cloud_utils.net_utils.sshconnection import CommandExitCodeException
from os import remove
from sys import exit
import threading
import time
import copy
from optparse import OptionParser, OptionValueError


parser = OptionParser('Instance Stress Test')

parser.add_option("-c", "--clc-ip", dest="clc_ip", default=None,
                  help="CLC IP")
parser.add_option("-p", "--password", dest="password", default='foobar',
                  help="clc ssh password")
parser.add_option("-u", "--user-count", dest="user_count", type="int", default=1,
                  help="Number of users")
parser.add_option("-v", "--vm-count", dest="vm_count", type="int", default=1,
                  help="Number of vms to run per user")
parser.add_option("-l", "--log-level", dest="log_level", default='DEBUG',
                  help="LOGLEVEL")
parser.add_option("--instance-timeout", dest="instance_timeout", type='int', default=180,
                  help="Seconds used as timeout in run-image/instance request")
parser.add_option("--prefix", dest="prefix", default='stressaccount',
                  help="Account name prefix")
parser.add_option("--account-start", dest="account_start", type='int', default=0,
                  help="Account name prefix + account-start value = accountname")
parser.add_option("--artifact-timeout", dest="artifact_timeout", type='int', default=300,
                  help="Seconds used as timeout waiting for vpc related artifacts on CLC")
parser.add_option("--freeze", dest="freeze", action='store_true', default=False,
                  help="Boolean to freeze test without cleaning up at end of test")

options, args = parser.parse_args()

count = options.user_count
vm_count = options.vm_count
log_level = str(options.log_level).upper()
clc_ip = options.clc_ip
instance_timeout = int(options.instance_timeout)
password = options.password
account_prefix = options.prefix
account_start = options.account_start
freeze_test = options.freeze
artifact_timeout = int(options.artifact_timeout)
users = []
userlock = threading.Lock()
errors = []
clc_times = {}
mido_md_times = {}
tc = TestController(clc_ip, password=password, log_level=log_level)
md = Midget(clc_ip, systemconnection=tc.sysadmin)
# Spread the love over all zones in the cloud...
max_zone_name = 0
zones = {}
zone_names = tc.sysadmin.get_all_cluster_names()
netprop = tc.sysadmin.get_property('{0}.cluster.networkmode'.format(zone_names[0]))
networkmode = netprop.value
for zone in zone_names:
    # Store the max name len for printing later
    if len(zone) > max_zone_name:
        max_zone_name = len(zone)
    # Init the zone with a 0 vm count
    zones[zone] = 0
i = 0
while i < vm_count:
    for zname in zones.iterkeys():
        if i >= vm_count:
            break
        zones[zname] += 1
        i += 1

print 'RUNNING TEST WITH ZONE VM COUNTS:'
for name, value in zones.iteritems():
    print "ZONE:{0} COUNT:{1}".format(name, value)
time.sleep(3)


def add_user(id):
    new_user = tc.create_user_using_cloudadmin(aws_user_name='admin',
                                               aws_account_name=account_prefix + str(x),
                                               service_connection=tc,
                                               log_level=log_level)
    print 'Add user found or created    :"{0}"'.format(new_user)
    new_user.ec2.log.set_stdout_loglevel(log_level)
    new_user.iam.log.set_stdout_loglevel(log_level)
    return new_user

def cleanup(user):
    try:
        user.ec2.terminate_instances()
        for key in user.ec2.test_resources['keypairs']:
            user.ec2.delete_keypair(key)
            remove(key.name + ".pem")
    except Exception as E:
        my_error = {'user':str(user), 'error':'{0}\nError:{1}'.format(get_traceback(), E)}
        with userlock:
            errors.append(my_error)

def get_clc_artifacts_for_vpc(user, vpc_ids, subnet_ids, addtime=0, interval=2,
                              timeout=artifact_timeout):
    start = time.time()
    elapsed = 0
    vpc_wait = copy.copy(vpc_ids)
    subnet_wait = copy.copy(subnet_ids)
    ret_dict = {}
    while elapsed < timeout and (vpc_wait or subnet_wait):
        user.log.info('Waiting for clc artifacts, elapsed:{0}/{1}, vpcs:{2}, subnets:{3}'
                          .format(int(elapsed + addtime), timeout, vpc_wait, subnet_wait))
        if vpc_wait:
            for vpc_id in vpc_ids:
                if vpc_id in vpc_wait:
                    for retry in xrange(0, 2):
                        try:
                            tc.sysadmin.clc_machine.sys('ps aux  | grep {0}'.format(str(vpc_id).lstrip('vpc-')), code=0)
                            if vpc_id in vpc_wait:
                                vpc_wait.remove(vpc_id)
                            ret_dict[vpc_id] = int(time.time() - start) + addtime
                            break
                        except Exception as E:
                            if isinstance(E, CommandExitCodeException):
                                pass
                            else:
                                tc.log.error("Error trying to fetch nginx ps for vpc:{0}".format(E))
                                time.sleep(1)
        if subnet_wait:
            for subnet_id in subnet_ids:
                if subnet_id in subnet_wait:
                    for retry in xrange(0, 2):
                        try:
                            tc.sysadmin.clc_machine.sys('ifconfig | grep {0}'.format(str(subnet_id).lstrip('subnet-')), code=0)
                            subnet_wait.remove(subnet_id)
                            ret_dict[subnet_id] = int(time.time() - start) + addtime
                            break
                        except Exception as E:
                            if isinstance(E, CommandExitCodeException):
                                pass
                            else:
                                tc.log.error("Error trying to fetch subnet veth:{0}".format(E))
                                time.sleep(1)

        elapsed = time.time() - start
        if elapsed < timeout and (vpc_wait or subnet_wait):
            time.sleep(interval)
        else:
            break
    for id in vpc_wait + subnet_wait:
        ret_dict[id] = 'Timedout:' + str(timeout + addtime)
    return ret_dict



def wait_for_mido_metadata_nat_rule(instances, addtime=0, interval=2, timeout=300):
    if networkmode != 'VPCMIDO':
        return {}
    start = time.time()
    elapsed = 0
    ret_dict = {}
    errbuff = ""
    waiting = copy.copy(instances)
    while waiting and elapsed < timeout:
        errbuff = ""
        for instance in instances:
            if instance in waiting:
                instance.update()
                rule = None
                try:
                    rule =  md.get_instance_bridge_port_metadata_nat_rule(instance.id)
                except Exception as E:
                    errbuff += ("{0}\nERROR fetching meta nat rule, elapsed:{1}, instance_state:{"
                                "2}, Error:{3}".format(get_traceback(),elapsed, instance.state, E))
                    if not int(elapsed) % 10:
                        md.log.error("ERROR fetching meta nat rule, elapsed:{0}, "
                                        "instance_state: {1}, Error:{2}"
                                        .format(elapsed, instance.state, E))
                if rule:
                    tc.log.info('Got MD rule for instance:{0} after elapsed{1}'
                                   .format(instance.id, elapsed))
                    try:
                        md.show_rules(rule)
                    except:
                        pass
                    elapsed = time.time() - start
                    ret_dict[instance.id] = elapsed + addtime
                    waiting.remove(instance)
                else:
                    if instance.state == 'terminated':
                        waiting.remove(instance)
                        elapsed = time.time() - start
                        ret_dict[instance.id] = "instance terminated after:{0}".format(elapsed)
        elapsed = time.time() - start
        if elapsed < timeout and waiting:
            time.sleep(interval)
    if errbuff:
        tc.log.error(errbuff)
    for instance in waiting:
        ret_dict[instance.id] = "Timed out waiting for MD after: {0}".format(int(elapsed))
    return ret_dict




def run_instance_with_user(user):
    my_error = {}
    err_string = ""
    instance_str = ""
    setattr(user, 'results', {})
    instances = []
    vpc_ids = []
    subnet_ids = []
    my_md_times = {}
    my_clc_times = None
    try:
        emi = user.ec2.get_emi(root_device_type='instance-store',
                               filters={'virtualization-type': 'hvm'},
                               basic_image=True)
        key = user.ec2.create_keypair_and_localcert('key_{0}_{1}_{2}'
                                                    .format(user.account_name, user.user_name,
                                                            int(time.time())))
        if not key:
            raise RuntimeError('Could not find or create key for account/user:{0}/{1}'
                               .format(user.account_name, user.user_name))
        group = user.ec2.add_group('group_{0}_{1}'.format(user.account_name, user.user_name))
        user.ec2.authorize_group(group=group, port=22, protocol='tcp')
        user.ec2.authorize_group(group=group, port=-1, protocol='icmp')
        start = time.time()
        for zone, count in zones.iteritems():
            instances += user.ec2.run_image(image=emi, keypair = key, group=group, min=count,
                                            max=count, zone=zone, timeout=instance_timeout,
                                            monitor_to_running=False, clean_on_fail=False,
                                            auto_connect=True)
        for i in instances:
            if not i.vpc_id in vpc_ids:
                vpc_ids.append(i.vpc_id)
            if not i.subnet_id in subnet_ids:
                subnet_ids.append(i.subnet_id)
        elapsed = int(time.time() - start)
        my_clc_times = get_clc_artifacts_for_vpc(user, vpc_ids, subnet_ids, addtime=elapsed)
        elapsed = int(time.time() - start)
        my_md_times = wait_for_mido_metadata_nat_rule(instances, addtime=elapsed)
        user.ec2.monitor_euinstances_to_running(instances, timeout=instance_timeout)
        instance_str = "\n".join("{0}:{1}".format(x.id, x.placement) for x in instances)
        my_results = {'user':str(user), 'result':'PASSED', 'instances':instance_str}
    except Exception as E:
        user.log.debug('{0}\nError:{1}'.format(get_traceback(), E))
        err_string += '{0}\nError:{1}'.format(get_traceback(), E)
        my_results = {'user':str(user), 'result': markup('FAILED', [1, 91]),
                      'instances':instance_str}
        try:
            for vpc_id in vpc_ids:
                cmd = 'ps aux  | grep {0}'.format(vpc_id)
                err_string += '\nCLC CMD#: {0}\n'.format(cmd)
                err_string += tc.sysadmin.clc_machine.sys(cmd, listformat=False)
        except Exception as E:
             tc.log.error('Failed to get vpc process info from clc:' + str(E))
        finally:
            err_string += "(CMD DONE)\n"
        try:
            for vpc_id in vpc_ids:
                cmd = 'ifconfig | grep {0}'.format(str(vpc_id).lstrip('vpc-'))
                err_string += '\nCLC CMD#: {0}\n'.format(cmd)
                err_string += tc.sysadmin.clc_machine.sys(cmd, listformat=False)
        except Exception as E:
             tc.log.error('Failed to get vpc ifconfig info from clc:' + str(E))
        finally:
            err_string += "(CMD DONE)\n"
        try:
            for subnet_id in subnet_ids:
                cmd = 'ifconfig | grep {0}'.format(str(subnet_id).lstrip('subnet-'))
                err_string += '\nCLC CMD#: {0}\n'.format(cmd)
                err_string += tc.sysadmin.clc_machine.sys(cmd, listformat=False)
        except Exception as E:
             tc.log.error('Failed to get subnet ifconfig info from clc:' + str(E))
        finally:
            err_string += "(CMD DONE)\n"
        for i in instances:
            try:
                cmd = "grep {0} /var/run/eucalyptus/eucanetd_vpc_instance_ip_map".format(i.id)
                err_string += '\nCLC CMD#: {0}\n'.format(cmd)
                err_string += tc.sysadmin.clc_machine.sys(cmd, listformat=False)
            except Exception as E:
                tc.log.error('Failed to get vpc nginx map info from clc:' + str(E))
            finally:
                err_string += "(CMD DONE)\n"
            err_string += i.get_cloud_init_info_from_console() + "\n"

    finally:
        try:
            for ins in instances:
                if ins and ins.ssh:
                    ins.ssh.close()
                ins = None
            instances = None
        except Exception as E:
            err_string += '{0}\nError:{1}'.format(get_traceback(), E)
            user.log.warn('')
        user.results = my_results
        with userlock:
            if err_string:
                my_error = {'user':str(user), 'error':err_string}
                errors.append(my_error)
            if my_clc_times:
                for key, value in my_clc_times.iteritems():
                    clc_times[key] = value
            if my_md_times:
                for key, value in my_md_times.iteritems():
                    mido_md_times[key] = value


for x in xrange(account_start, count + account_start):
    users.append(add_user(x))
tc.log.info('Done Creating Users')
upt = PrettyTable(['ACCOUNT', 'NAME', 'ACCT_ID'])
for user in users:
    upt.add_row([user.account_name, user.user_name, user.account_id])
upt.sortby='ACCOUNT'

print "\n{0}\n".format(upt)
time.sleep(5)

def print_results():
    errpt = PrettyTable(['USER', 'ERROR'])
    errpt.max_width['USER'] = 20
    errpt.max_width['ERROR'] = 100
    errpt.hrules = 1
    errpt.align = 'l'
    for err in errors:
        errpt.add_row([err.get('user'), err.get('error')])
    print "\n{0}\n".format(errpt)
    tc.log.info('User - Thread Completed! Current Results....')
    pt = PrettyTable(['ACCOUNT', 'USER', 'RESULT', 'INSTANCES'])
    pt.align = 'l'
    pt.max_width['INSTANCES'] = 11 + max_zone_name + 1
    pt.sortby = 'ACCOUNT'
    for user in users:
        if user.results:
            res = user.results
            pt.add_row([user.account_name, user.user_name, res.get('result'), res.get('instances')])
    print "\n{0}\n".format(pt)
    tt = PrettyTable(['CLC VETH', 'CREATE TIME'])
    tt.align = 'l'
    tt.hrules = 1
    for key, value in clc_times.iteritems():
        tt.add_row([key, value])
    print "\n{0}\n".format(tt)
    mdt = PrettyTable(['INSTANCE', 'MIDO MD RULE TIME'])
    mdt.align = 'l'
    mdt.hrules = 1
    for key, value in mido_md_times.iteritems():
        mdt.add_row([key, value])
    print "\n{0}\n".format(mdt)


threads = []
for user in users:
    t = threading.Thread(target=run_instance_with_user, args=(user, ))
    t.start()
    threads.append(t)
for t in threads:
    t.join()
    print_results()

if not freeze_test:
    threads = []
    for user in users:
        t = threading.Thread(target=cleanup, args=(user, ))
        t.start()
        threads.append(t)
    for t in threads:
        t.join()

print_results()
tc.log.info('User - All threads have completed')

if errors:
    exit(1)
else:
    exit(0)



#!/usr/bin/env python
from __future__ import division

from cStringIO import StringIO
import tempfile

import os
from concurrent.futures.thread import ThreadPoolExecutor
from math import ceil

from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from nephoria.testcontroller import TestController
import copy
import time


class OSGConcurrentTests(CliTestRunner):

    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    _DEFAULT_CLI_ARGS['buckets'] = {
        'args': ['--buckets'],
        'kwargs': {'dest': 'buckets', 'help': 'Number of buckets', 'default': 5, 'type': int}
    }

    _DEFAULT_CLI_ARGS['objects'] = {
        'args': ['--objects'],
        'kwargs': {'dest': 'objects', 'help': 'Number of objects', 'default': 5, 'type': int}
    }

    _DEFAULT_CLI_ARGS['threads'] = {
        'args': ['--threads'],
        'kwargs': {'dest': 'threads', 'help': 'Number of threads', 'default': 3, 'type': int}
    }

    _DEFAULT_CLI_ARGS['object_size'] = {
        'args': ['--object-size'],
        'kwargs': {'dest': 'object_size', 'help': 'Size of the objects to upload', 'default': 5, 'type': int}
    }

    _DEFAULT_CLI_ARGS['mpu_threshold'] = {
        'args': ['--mpu-threshold'],
        'kwargs': {'dest': 'mpu_threshold', 'default': 5120, 'type': int,
                   'help': 'Multipart upload is used when the object size is bigger than the mpu-threshold'
                           'value in Kilobyte. Any value less than 5120KB will result single file upload. '
                           'Default value is used when not passed as an argument.'}
    }

    bucket_list = []
    temp_files = []
    total_put_latency = 0
    total_get_latency = 0
    total_del_latency = 0

    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            tc = TestController(self.args.clc,
                                password=self.args.password,
                                clouduser_name=self.args.test_user,
                                clouduser_account=self.args.test_account,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    @property
    def user(self):
        user = getattr(self, '__user', None)
        if not user:
            try:
                user = self.tc.get_user_by_name(aws_account_name=self.args.test_account,
                                                aws_user_name=self.args.test_user)
            except:
                user = self.tc.create_user_using_cloudadmin(aws_account_name=self.args.test_account,
                                                            aws_user_name=self.args.test_user)
            setattr(self, '__user', user)
        return user

    @property
    def bucket_prefix(self):
        bucket_prefix = getattr(self, '__bucket_prefix', None)
        if not bucket_prefix:
            bucket_prefix = "nephoria-bucket-" + str(int(time.time()))
        return bucket_prefix

    @bucket_prefix.setter
    def bucket_prefix(self, value):
        setattr(self, '__bucket_prefix', value)

    def create_file(self, size_in_kb, file_name="nephoria-object"):
        temp_file = tempfile.NamedTemporaryFile(mode='w+b', prefix=file_name)
        self.temp_files.append(temp_file)
        temp_file.write(os.urandom(1024 * size_in_kb))
        return temp_file.name

    def single_upload(self, bucket, key_name, file_path):
        key = bucket.new_key(key_name)
        key.set_contents_from_filename(file_path)
        self.log.debug("Uploaded key '" + key_name + "' to bucket '" + bucket.name + "'")
        return key

    def multipart_upload(self, bucket, key_name, eufile):
        part_size = 1024 * self.args.mpu_threshold
        eufile.seek(0, os.SEEK_END)
        eufile_size = eufile.tell()
        num_parts = int(ceil(eufile_size / part_size))

        mpu = bucket.initiate_multipart_upload(key_name)
        self.log.debug("Initiated MPU. Using MPU Id: " + mpu.id)

        for i in range(num_parts):
            start = part_size * i
            file_part = open(eufile.name, 'rb')
            file_part.seek(start)
            data = file_part.read(part_size)
            file_part.close()
            mpu.upload_part_from_file(StringIO(data), i + 1)
            self.log.debug("Uploaded part " + str(i + 1) + " of '" + key_name + "' to bucket '" + bucket.name + "'")
        self.log.debug("Completing multipart upload of '" + key_name + "' to bucket '" +
                   bucket.name + "'" + " using mpu id: " + mpu.id)
        mpu.complete_upload()
        self.log.debug("Completed multipart upload of '" + key_name + "' to bucket '" + bucket.name + "'")

    def put_get_check(self, bucket_name, key_name, eu_file):
        """
        PUT objects, GET objects and then verify objects with object hash
        5MB is a hard-coded limit for MPU in OSG
        """
        bucket = self.tc.admin.s3.get_bucket_by_name(bucket_name)
        if (os.path.getsize(eu_file.name) > (5 * 1024 * 1024)) and (self.args.mpu_threshold >= (5 * 1024)):
            self.multipart_upload(bucket, key_name, eu_file)
        else:
            upload_time = self.time_to_exec(self.single_upload, bucket, key_name, eu_file.name)
            with open('osg_perf.log', 'a') as f:
                f.write('PUT\t' + str(upload_time) + '\n')
        return True

    def time_to_exec(self, method, *args, **kwargs):
        start_time = time.time()
        method_name = method.__name__
        try:
            result = method(*args, **kwargs)
        except Exception as e:
            self.log.error(e)
            end_time = time.time()
            total_time = end_time - start_time
            self.log.error("Failed to run method: " + method_name)
        else:
            end_time = time.time()
            total_time = end_time - start_time
        return total_time

    def create_buckets(self, num):
        for i in range(num):
            bucket_name = self.bucket_prefix + '-' + str(i)
            self.log.debug('creating bucket: ' + bucket_name)
            self.bucket_list.append(bucket_name)
            bucket = self.tc.admin.s3.connection.create_bucket(bucket_name)

        self.log.debug(self.tc.admin.s3.connection.get_all_buckets())

    def get_content(self, key):
        self.log.debug("Getting content as string for: " + key.name)
        content = key.get_contents_as_string()



    def put_objects(self, bucket_name, key_name, eu_file=None):
        """
        Args:
            bucket_name: existing bucket_name to put objects
            key_name: name of the key
            eu_file: file to put into bucket
        """
        bucket = self.tc.admin.s3.get_bucket_by_name(bucket_name)

        if (os.path.getsize(eu_file.name) > (5 * 1024 * 1024)) and (self.args.mpu_threshold >= (5 * 1024)):
            self.multipart_upload(bucket, key_name, eu_file)
        else:
            upload_time = self.time_to_exec(self.single_upload, bucket, key_name, eu_file.name)
            self.total_put_latency = self.total_put_latency + upload_time
            with open('osg_perf.log', 'a') as f:
                f.write('PUT\t\t' + str(upload_time) + '\n')
            return True

    def test1_concurrent_upload(self):
        with open('osg_perf.log', 'w') as f:
            f.write(str('OPS\t\t   Time   ') + '\n')
            f.write(str('---\t\t----------') + '\n')
        self.log.debug("Creating buckets..")
        self.create_buckets(self.args.buckets)

        self.log.debug("Creating object of " + str(self.args.object_size) + "KB")
        eu_file = open(self.create_file(self.args.object_size))

        thread_pool = []
        with ThreadPoolExecutor(max_workers=self.args.threads) as executor:
            for bucket_name in self.bucket_list:
                for k in range(self.args.objects):
                    thread_pool.append(executor.submit(self.put_objects,
                                                       bucket_name=bucket_name,
                                                       key_name=eu_file.name + '-' + str(k),
                                                       eu_file=eu_file))
        lock_time = 2
        self.log.debug("len(thread_pool): " + str(len(thread_pool)))
        while len(thread_pool) < (self.args.buckets * self.args.objects):
            self.log.debug("len(thread_pool): " + str(len(thread_pool)))
            self.log.warning("Uncanny lock, sleeping for " + str(lock_time) + " seconds.")
            time.sleep(lock_time)

        for tp in thread_pool:
            try:
                if not tp.result():
                    self.log.error("[CRITICAL] failed upload in thread")
            except Exception as e:
                self.log.error("Found exception in thread-pool: " + e.message)

    def get_objects(self, key):
        download_time = self.time_to_exec(self.get_content, key)
        self.total_get_latency = self.total_get_latency + download_time
        with open('osg_perf.log', 'a') as f:
            f.write('GET\t\t' + str(download_time) + '\n')

    def test2_get_objects(self):
        get_thread_pool = []
        with ThreadPoolExecutor(max_workers=self.args.threads) as executor:
            for bucket_name in self.bucket_list:
                bucket = self.tc.admin.s3.get_bucket_by_name(bucket_name)
                max_keys = 10
                keys = bucket.get_all_keys(max_keys=max_keys)
                for key in keys:
                    get_thread_pool.append(executor.submit(self.get_objects, key))
                while keys.next_marker:
                    self.log.debug("found keys.next_marker: " + keys.next_marker)
                    keys = bucket.get_all_keys(marker=keys.next_marker)
                    for key in keys:
                        get_thread_pool.append(executor.submit(self.get_objects, key))
        self.log.debug("len(get_thread_pool): " + str(len(get_thread_pool)))

        lock_time = 2
        while len(get_thread_pool) < (self.args.buckets * self.args.objects):
            self.log.debug("len(get_thread_pool): " + str(len(get_thread_pool)))
            self.log.warning("Uncanny lock, sleeping for " + str(lock_time) + " seconds.")
            time.sleep(lock_time)

    def delete_key(self, key):
        self.log.debug('deleting key: ' + key.name)
        delete_time = self.time_to_exec(key.delete)
        self.total_del_latency = self.total_del_latency + delete_time
        with open('osg_perf.log', 'a') as f:
            f.write('DEL\t\t' + str(delete_time) + '\n')
        return True

    def test3_delete_objects(self):
        clean_thread_pool = []

        with ThreadPoolExecutor(max_workers=self.args.threads) as executor:
            for bucket_name in self.bucket_list:
                bucket = self.tc.admin.s3.get_bucket_by_name(bucket_name)
                max_keys = 10
                keys = bucket.get_all_keys(max_keys=max_keys)
                for key in keys:
                    clean_thread_pool.append(executor.submit(self.delete_key, key))
                while keys.next_marker:
                    self.log.debug("found keys.next_marker: " + keys.next_marker)
                    keys = bucket.get_all_keys(marker=keys.next_marker)
                    for key in keys:
                        clean_thread_pool.append(executor.submit(self.delete_key, key))

        self.log.debug("len(clean_thread_pool): " + str(len(clean_thread_pool)))
        lock_time = 2
        while len(clean_thread_pool) < (self.args.buckets * self.args.objects):
            self.log.debug("len(clean_thread_pool): " + str(len(clean_thread_pool)))
            self.log.warning("Uncanny lock, sleeping for " + str(2) + " seconds.")
            time.sleep(lock_time)

        for ctp in clean_thread_pool:
            try:
                if not ctp.result():
                    self.log.error("[CRITICAL] failed delete in thread")
            except Exception as e:
                self.log.error("Found exception in clean_thread_pool: " + e.message)

        for bucket_name in self.bucket_list:
            self.tc.admin.s3.connection.delete_bucket(bucket_name)
        for tf in self.temp_files:
            tf.close()

    def test4_calculate_average_latency(self):
        with open('osg_perf.log', 'a') as f:
            f.write('\n\n')
            f.write('  Average Latency  ' + '\n')
            f.write('-------------------' + '\n')
        avg_put = self.total_put_latency / (self.args.objects * self.args.buckets)
        with open('osg_perf.log', 'a') as f:
            f.write('Avg PUT\t\t' + str(avg_put) + '\n')
        avg_get = self.total_get_latency / (self.args.objects * self.args.buckets)
        with open('osg_perf.log', 'a') as f:
            f.write('Avg GET\t\t' + str(avg_get) + '\n')
        avg_del = self.total_del_latency / (self.args.objects * self.args.buckets)
        with open('osg_perf.log', 'a') as f:
            f.write('Avg DEL\t\t' + str(avg_del) + '\n')

    def clean_method(self):
        pass

if __name__ == "__main__":
    test = OSGConcurrentTests()
    test_result = test.run()
    exit(test_result)



#!/usr/bin/python


import testcase_utils
import re
import random
import string

__version__ = '0.4.3.0'
__DEFAULT_API_VERSION__ = '2015-10-01'


def handle_timeout(self, signum, frame):
    raise testcase_utils.TimeoutFunctionException()

def grep(self, string, list):
    """ Remove the strings from the list that do not match the regex string"""
    expr = re.compile(string)
    return filter(expr.search,list)

def render_file_template(src, dest, **kwargs):
    from cloud_utils import file_utils
    return file_utils.render_file_template(src, dest, **kwargs)

def id_generator(size=6, chars=None):
    """Returns a string of size with random charachters from the chars array.
         size    Size of string to return
         chars   Array of characters to use in generation of the string
    """
    chars = chars or (string.ascii_uppercase + string.ascii_lowercase  + string.digits)
    return ''.join(random.choice(chars) for x in range(size))

class CleanTestResourcesException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: Vic Iglesias vic.iglesias@eucalyptus.com
#

from boto.cloudformation import CloudFormationConnection
from boto.cloudformation.stack import Stack
from boto.exception import BotoServerError
import time
import json
import os
import re
import urllib
from prettytable import PrettyTable
from nephoria.baseops.botobaseops import BotoBaseOps

class CFNops(BotoBaseOps):
    SERVICE_PREFIX = 'cloudformation'
    CONNECTION_CLASS = CloudFormationConnection
    EUCARC_URL_NAME = 'cloudformation_url'

    def setup_resource_trackers(self):
        ## add test resource trackers and cleanup methods...
        self.test_resources["stacks"] = self.test_resources.get('stacks', [])
        self.test_resources_clean_methods["stacks"] = None

    def create_stack(self, stack_name, template_body, template_url=None, parameters=None,
                     *args, **kwargs):
        self.log.info("Creating stack: {0}".format(stack_name))
        arn = self.connection.create_stack(stack_name, template_body,
                                           template_url=template_url,
                                           parameters=parameters, *args, **kwargs)
        for x in xrange(0, 5):
            stacks = self.connection.describe_stacks(arn)
            if stacks:
                self.test_resources["stacks"].append(stacks[0])
                return stacks[0]
            time.sleep(2 * x)

    create_stack.__doc__ = CloudFormationConnection.create_stack.__doc__

    def validate_template(self, template_body, template_url=None, *args, **kwargs):
        self.log.info("Validating template: {0}".format(template_body))
        return self.connection.validate_template(template_body,
                                                 template_url=template_url,
                                                 *args, **kwargs) 

    validate_template.__doc__ = CloudFormationConnection.validate_template.__doc__

    def delete_stack(self, stack_name_or_id, *args, **kwargs):
        if not isinstance(stack_name_or_id, basestring):
            stack_name_or_id = stack_name_or_id.stack_id
        self.log.info("Deleting stack: {0}".format(stack_name_or_id))
        return self.connection.delete_stack(stack_name_or_id, *args, **kwargs)

    delete_stack.__doc__ = CloudFormationConnection.delete_stack.__doc__

    def delete_all_stacks(self, timeout=360, poll_sleep=10):
        """
        Deletes all stacks.

        Args:
            timeout: default= 60
            poll_sleep: 10 seconds

        Returns: list of stacks, empty list if succeeded

        """
        stacks = self.describe_stacks()
        poll_count = timeout / poll_sleep
        if len(stacks) > 0:
            for i in stacks:
                self.log.debug("Deleting Stack: {0}".format(i))
                self.delete_stack(i)
            for _ in range(poll_count):
                time.sleep(poll_sleep)
                stacks = self.describe_stacks()
                if len(stacks) == 0:
                    break
                for i in stacks:
                    if i in stacks:
                        self.delete_stack(i)
        stacks = self.describe_stacks()
        return stacks

    def describe_stacks(self, stack_names_or_ids=None, *args, **kwargs):
        if stack_names_or_ids and not isinstance(stack_names_or_ids, list):
            stack_names_or_ids = [stack_names_or_ids]
        self.log.info("Describing stack: {0}".format(stack_names_or_ids))
        return self.connection.describe_stacks(stack_names_or_ids, *args, **kwargs)

    describe_stacks.__doc__ = CloudFormationConnection.describe_stacks.__doc__




#!/usr/bin/python

from logging import DEBUG, NOTSET
from cloud_utils.log_utils.eulogger import Eulogger
from cloud_utils.log_utils import markup, get_traceback
from cloud_utils.file_utils.eucarc import Eucarc
from nephoria import CleanTestResourcesException
from urlparse import urlparse


AWSRegionData = {
    'us-east-1': 'us-east-1.amazonaws.com',
    'us-west-1': 'us-west-1.amazonaws.com',
    'us-west-2': 'us-west-2.amazonaws.com',
    'eu-west-1': 'eu-west-1.amazonaws.com',
    'ap-northeast-1': 'ap-northeast-1.amazonaws.com',
    'ap-southeast-1': 'ap-southeast-1.amazonaws.com'}

class BaseOps(object):
    # The key name this ops class uses to look up it's service url value (ie EC2_URL, S3_URL, etc)
    EUCARC_URL_NAME = None
    # The service prefix used with the region (ie ec2, iam, s3, etc)
    SERVICE_PREFIX = None
    # The underlying class used to connect to the cloud (ie boto.VPCConnection)
    CONNECTION_CLASS = None

    def __init__(self, eucarc=None, credpath=None, service_url=None, aws_access_key_id=None,
                 aws_secret_access_key=None, is_secure=False, port=None, host=None,
                 region=None, connection_debug=0, path=None, validate_certs=True,
                 test_resources=None, logger=None, log_level=None, user_context=None,
                 session=None, api_version=None, verbose_requests=None):
        if self.EUCARC_URL_NAME is None:
            raise NotImplementedError('EUCARC_URL_NAME not set for this class:"{0}"'
                                      .format(self.__class__.__name__))
        if self.SERVICE_PREFIX is None:
            raise NotImplementedError('Service Prefix has not been defined for this class:"{0}"'
                                      .format(self.__class__.__name__))
        init_kwargs = locals()
        init_kwargs.__delitem__('self')
        self._session = session
        self.connection = None
        self.service_host = None
        self.service_port = None
        self.service_path = None
        # Store info about created resources and how to clean/delete them for this ops connection
        self.test_resources_clean_methods = {}
        self.test_resources = test_resources or {}
        # Store the user context for this connection if provided
        self._user_context = user_context
        if not region and self._user_context:
            region = self._user_context.region
        self.service_region = region
        # Create the logger for this ops connection
        if log_level is None:
            log_level = DEBUG
        def get_logger_context():
            host = self.service_host or ""
            context = ""
            try:
                if self._user_context:
                    if self._user_context._user_name and self._user_context._account_name:

                        context = "({0}:{1})".format(self._user_context.user_name,
                                                     self._user_context.account_name)
                    elif self._user_context.access_key:
                        context = "(AK:{0}...)".format(self._user_context.__access_key[5:])
            except Exception as LE:
                print 'error fetching user context: "{0}"'.format(LE)
            if not context:
                if aws_access_key_id:
                    context = "(AK:{0}...)".format(aws_access_key_id[5:])
                else:
                    context = "()"
            return context
        if not logger:
            logger = Eulogger("{0}{1}".format(self.__class__.__name__, get_logger_context()),
                              stdout_level=log_level)
        self.log = logger
        self.log.debug('Creating ops: {0}'.format(self.__class__.__name__))
        self.log.set_stdout_loglevel(log_level)
        # Store the runtime configuration for this ops connection
        if not eucarc:
            if credpath:
                eucarc = Eucarc(filepath=credpath)
            else:
                eucarc = Eucarc()
        self.eucarc = eucarc
        # Set the connection params...
        self._try_verbose = verbose_requests
        self._is_secure = is_secure
        if aws_secret_access_key:
            self.eucarc.aws_secret_key = aws_secret_access_key
        if aws_access_key_id:
            self.eucarc.aws_access_key = aws_access_key_id
        self._service_url = service_url
        if not (host or port or path):
            if self.service_url:
                urlp = urlparse(self.service_url)
                host = host or urlp.hostname
                port = port or urlp.port or 8773
                path = path or urlp.path
        self.service_host = host
        self.service_port = port
        self.service_path = path
        self.service_region = region
        # Build out kwargs used to create service connection/client
        # Pass all the args/kwargs provided at init to the create_connection_kwargs method for the
        # ops class to use to build it's kwargs as needed.
        self._connection_kwargs = self.create_connection_kwargs(**init_kwargs)
        self.connect(verbose=connection_debug)
        # Remaining setup...
        self.setup()

    def __repr__(self):
        return "{0}:{1}:{2}".format(self.__class__.__name__, self.service_region,
                                    self.SERVICE_PREFIX)

    def connect(self, verbose=False,  connection_kwargs=None):
        """
        Verify the required params have been set, and connect the underlying connection class.

        :param verbose: Dump debug output about the connection
        :param connection_kwargs: options dict containing kwargs used when creating the
                                  underlying connection
        """
        if self.CONNECTION_CLASS is None:
            raise NotImplementedError('Connection Class has not been defined for this class:"{0}"'
                                      .format(self.__class__.__name__))
        if connection_kwargs:
            self._connection_kwargs = connection_kwargs
        # Remove and kwargs which are not part of the service connection class creation method
        self._clean_connection_kwargs()
        required = []
        for key, value in self._connection_kwargs.iteritems():
            if value is None:
                required.append(key)
        if required:
            self.show_connection_kwargs(connection_kwargs=connection_kwargs)
            raise ValueError('{0}: Required Connection parameters were None: "{1}"'
                             .format(self.__class__.__name__, ", ".join(required)))
        #### Init connection...
        if verbose:
            self.show_connection_kwargs()
        try:
            # Remove any kwargs that are not applicable to this connection class
            # For example 'region' may not be applicable to services such as 'IAM'
            connection_keys = self._connection_kwargs.keys()
            for ckey in connection_keys:
                if ckey not in self.CONNECTION_CLASS.__init__.__func__.__code__.co_varnames:
                    self.log.debug('Arg "0" not found in "{1}.init()", removing kwarg '
                                   'connection args.'.format(ckey, self.CONNECTION_CLASS.__name__))
                    self._connection_kwargs.__delitem__(ckey)
            self.connection = self.CONNECTION_CLASS(**self._connection_kwargs)
        except:
            self.show_connection_kwargs()
            raise

    def create_connection_kwargs(self, **kwargs):
        self._connection_kwargs = kwargs


    def setup(self):
        self.setup_resource_trackers()

    @property
    def service_name(self):
        return self.SERVICE_PREFIX

    @property
    def service_url(self):
        if not self._service_url:
            url = getattr(self.eucarc, self.EUCARC_URL_NAME, None)
            if url:
                self._service_url = url
            else:
                if self._is_secure:
                    prefix = 'https'
                elif self.service_host:
                    prefix = 'http'
                    url = "{0}://{1}:{2}{3}".format(prefix,
                                                self.service_host,
                                                self.service_port or "",
                                                self.service_path or "")
                    return url
        return self._service_url

    @property
    def session(self):
        return self._session

    @session.setter
    def session(self, value):
        self._session = value

    def enable_connection_debug(self, level=DEBUG, format_string=None):
        pass

    def disable_connection_debug(self, level=NOTSET):
        pass

    @property
    def _use_verbose_requests(self):
        if self._try_verbose is None:
            self._try_verbose = False
            if self.eucarc:
                account = getattr(self.eucarc, 'aws_account_name', None)
                user = getattr(self.eucarc, 'aws_user_name', None)
                if account == 'eucalyptus' and user == 'admin':
                    self._try_verbose = True
        return self._try_verbose

    @_use_verbose_requests.setter
    def _use_verbose_requests(self, value):
        if value is None or isinstance(value, bool):
            self._try_verbose = value
            return
        raise ValueError('Only bool or None type supported for "_use_verbose_requests". '
                         'Got: "{0}/{1}"'.format(value, type(value)))

    def _clean_connection_kwargs(self, connection_kwargs=None, connection_method=None):
        # Remove any kwargs from self_connection_kwargs that are not applicable
        # to self.CONNECTION_CLASS
        if connection_kwargs is None:
            connection_kwargs = self._connection_kwargs or {}
        if connection_method is None:
            connection_method = self.CONNECTION_CLASS.__init__
        varnames = connection_method.__func__.func_code.co_varnames
        keys = connection_kwargs.keys()
        for key in keys:
            if key not in varnames:
                del connection_kwargs[key]
        return connection_kwargs


    def show_connection_kwargs(self, connection_kwargs=None):
        if connection_kwargs is None:
            connection_kwargs = self._connection_kwargs
        print connection_kwargs
        debug_buf = 'Current "{0}" connection kwargs for\n'.format(self.__class__.__name__)
        for key, value in connection_kwargs.iteritems():
            debug_buf += "{0}{1}{2}\n".format(str(key).ljust(30), " -> ", value)
        self.log.debug(debug_buf)


    def setup_resource_trackers(self):
        """
        Allows each ops class to track resources created by this ops class, as well as the
        method(s) to user per resource to type to clean/remove them.
        For example an ec2_ops class may create 'instances' and later can register a 'terminate()'
        method to delete these upon exit.
        """
        raise NotImplementedError('ERROR: {0} has not implemented resource tracking method. '
                                  '"test_resources" and "test_resources_clean_methods" should be '
                                  'setup here.'
                                  .format(self.__class__.__name__))

    def clean_all_test_resources(self):
        fault_buf = ""
        for resource_name, resource_list in self.test_resources.iteritems():
            clean_method = self.test_resources_clean_methods.get(resource_name, None)
            if clean_method:
                try:
                    try:
                        clean_method_name = clean_method.__func__.__name__
                    except:
                        clean_method_name = str(clean_method)
                    self.log.debug('Attempting to clean test resources of type:"{0}", '
                                   'method:"{1}", artifacts:"{2}"'
                                   .format(resource_name, clean_method_name, resource_list))
                    clean_method(resource_list)
                except Exception as E:
                    fault_buf += "{0}\n{1}\n".format(get_traceback(),
                                                     markup('Error while attempting to remove '
                                                            'test resource type:"{0}", '
                                                            'error:"{1}"'
                                                            .format(resource_name, E)))
        if fault_buf:
            raise CleanTestResourcesException(fault_buf)


